#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Formatter to de/format omor style analyses for omrfi."""

# (c) Omorfi contributors <omorfi-devel@groups.google.com> 2015–2018
# see AUTHORS file in top-level dir of this project, or
# <https://github.com/flammie/omorfi/wiki/AUTHORS>

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# utils to format apertium style data from omorfi database values

import re

from ..error_logging import fail_formatting_missing_for, just_fail
from .formatter import Formatter
from ..string_manglers import egrep2xerox, lexc_escape, regex_delete_surface


class OmorFormatter(Formatter):
    """Converts Omorfi's default formats for strings."""

    ## Omorfi multichars for all omorfi analysers
    common_multichars = {
        '[ABBR=ABBREVIATION]',
        '[ABBR=ACRONYM]',
        "[BOUNDARY=CLAUSE]",
        '[BOUNDARY=COMPOUND]',
        "[BOUNDARY=SENTENCE]",
        '[CASE=ABE]',
        '[CASE=ABL]',
        '[CASE=ACC]',
        '[CASE=ADE]',
        '[CASE=ALL]',
        '[CASE=COM]',
        '[CASE=ELA]',
        '[CASE=ESS]',
        '[CASE=GEN]',
        '[CASE=ILL]',
        '[CASE=INE]',
        '[CASE=INS]',
        '[CASE=LAT]',
        '[CASE=NOM]',
        '[CASE=PAR]',
        '[CASE=TRA]',
        '[CLIT=HAN]',
        '[CLIT=KA]',
        '[CLIT=KAAN]',
        '[CLIT=KIN]',
        '[CLIT=KO]',
        '[CLIT=PA]',
        '[CLIT=S]',
        '[CMP=CMP]',
        '[CMP=POS]',
        '[CMP=SUP]',
        '[COMPOUND_FORM=MISSING-]',
        '[COMPOUND_FORM=OMIT]',
        '[COMPOUND_FORM=S]',
        '[CONJ=ADVERBIAL]',
        '[CONJ=COMPARATIVE]',
        '[CONJ=COORD]',
        '[DRV=IN]',
        '[DRV=IN²]',
        '[DRV=ISA]',
        '[DRV=INEN]',
        '[DRV=NEN]',
        '[DRV=JA]',
        '[DRV=LAINEN]',
        '[DRV=LA]',
        '[DRV=LLINEN]',
        '[DRV=MA]',
        '[DRV=MAISILLA]',
        '[DRV=MATON]',
        '[DRV=MINEN]',
        '[DRV=MAINEN]',
        '[DRV=LAINEN]',
        '[DRV=MPI]',
        '[DRV=NUT]',
        '[DRV=NUT]',
        '[DRV=OI]',
        '[DRV=HKO]',
        '[DRV=S]',
        '[DRV=STI]',
        '[DRV=TAR]',
        '[DRV=TATTAA]',
        '[DRV=TATUTTAA]',
        '[DRV=TUTTAA]',
        '[DRV=TAVA]',
        '[DRV=TON]',
        '[DRV=TSE]',
        '[DRV=TTAA]',
        '[DRV=TTAIN]',
        '[DRV=TU]',
        '[DRV=U]',
        '[DRV=UUS]',
        '[DRV=VA]',
        '[DRV=VS]',
        '[FILTER=NO_PROC]',
        '[GUESS=COMPOUND]',
        '[GUESS=DERIVE]',
        '[INF=A]',
        '[INF=E]',
        '[INF=MA]',
        '[INF=MAISILLA]',
        '[INF=MINEN]',
        "[LEX=ABE]",
        "[LEX=ABL]",
        "[LEX=ADE]",
        "[LEX=ALL]",
        "[LEX=ELA]",
        "[LEX=ILL]",
        "[LEX=INE]",
        "[LEX=INS]",
        "[LEX=PAR]",
        "[LEX=STI]",
        "[LEX=GEN]",
        "[LEX=LAT]",
        "[LEX=LOC]",
        "[LEX=SEP]",
        "[LEX=TTAIN]",
        '[MOOD=COND]',
        '[MOOD=EVNV]',
        '[MOOD=IMPV]',
        '[MOOD=INDV]',
        '[MOOD=INDV][TENSE=PAST]',
        '[MOOD=INDV][TENSE=PRESENT]',
        '[MOOD=OPT]',
        '[MOOD=POTN]',
        '[NEG=CON]',
        '[NUM=PL]',
        '[NUM=SG]',
        '[NUMTYPE=CARD]',
        '[NUMTYPE=ORD]',
        '[NUMTYPE=FRAC]',
        '[NUMTYPE=MULT]',
        '[PCP=AGENT]',
        '[PCP=NEG]',
        '[PCP=NUT]',
        '[PCP=VA]',
        '[PERS=PE4]',
        '[PERS=PL1]',
        '[PERS=PL2]',
        '[PERS=PL3]',
        '[PERS=SG0]',
        '[PERS=SG1]',
        '[PERS=SG2]',
        '[PERS=SG3]',
        '[POSITION=PREFIX]',
        '[POSITION=SUFFIX]',
        '[POSS=3]',
        '[POSS=PL1]',
        '[POSS=PL2]',
        '[POSS=PL3]',
        '[POSS=SG1]',
        '[POSS=SG2]',
        '[POSS=SG3]',
        '[PRONTYPE=DEM]',
        '[PRONTYPE=IND]',
        '[PRONTYPE=INT]',
        '[PRONTYPE=PRS]',
        '[PRONTYPE=RCP]',
        '[PRONTYPE=REL]',
        '[PRONTYPE=REC]',
        '[PROPER=ARTWORK]',
        '[PROPER=CULTGRP]',
        '[PROPER=EVENT]',
        '[PROPER=FIRST]',
        '[PROPER=GEO]',
        '[PROPER=LAST]',
        '[PROPER=MEDIA]',
        '[PROPER=MISC]',
        '[PROPER=ORG]',
        '[PROPER=PRODUCT]',
        '[SEM=COUNTRY]',
        '[SEM=CURRENCY]',
        '[SEM=EVENT]',
        '[SEM=FEMALE]',
        '[SEM=GEO]',
        '[SEM=INHABITANT]',
        '[SEM=LANGUAGE]',
        '[SEM=MALE]',
        '[SEM=MEASURE]',
        '[SEM=MEDIA]',
        '[SEM=ORG]',
        '[SEM=POLIT]',
        '[SEM=TIME]',
        '[SEM=TITLE]',
        '[STYLE=ARCHAIC]',
        '[STYLE=DIALECTAL]',
        '[STYLE=NONSTANDARD]',
        '[STYLE=RARE]',
        '[SUBCAT=ARROW]',
        '[SUBCAT=BRACKET]',
        "[SUBCAT=BRACKET][POSITION=FINAL]",
        "[SUBCAT=BRACKET][POSITION=INITIAL]",
        '[SUBCAT=COMMA]',
        '[SUBCAT=CONJUNCTION]',
        '[SUBCAT=CURRENCY]',
        '[SUBCAT=DASH]',
        "[SUBCAT=DASH]",
        '[SUBCAT=DECIMAL]',
        '[SUBCAT=DEMONSTRATIVE]',
        '[SUBCAT=DIGIT]',
        '[SUBCAT=FINAL]',
        '[SUBCAT=INITIAL]',
        '[SUBCAT=INTERJECTION]',
        '[SUBCAT=INTERROGATIVE]',
        '[SUBCAT=MATH]',
        '[SUBCAT=NEG]',
        "[SUBCAT=NEG]",
        '[SUBCAT=OPERATION]',
        '[ADPTYPE=POST]',
        '[SUBCAT=PREFIX]',
        '[ADPTYPE=PREP]',
        '[SUBCAT=QUALIFIER]',
        '[SUBCAT=QUANTIFIER]',
        '[SUBCAT=QUOTATION]',
        "[SUBCAT=QUOTATION][POSITION=FINAL]",
        "[SUBCAT=QUOTATION][POSITION=INITIAL]",
        '[SUBCAT=REFLEXIVE]',
        '[SUBCAT=RELATION]',
        '[SUBCAT=ROMAN]',
        '[SUBCAT=SPACE]',
        '[SUBCAT=SUFFIX]',
        '[TENSE=PAST]',
        '[TENSE=PRESENT]',
        '[UPOS=ADJ]',
        '[UPOS=ADP]',
        '[UPOS=ADV]',
        '[UPOS=AUX]',
        '[UPOS=DET]',
        '[UPOS=INTJ]',
        '[UPOS=CCONJ]',
        '[UPOS=SCONJ]',
        '[UPOS=SCONJ][CONJ=COMPARATIVE]',
        '[UPOS=SCONJ][CONJ=ADVERBIAL]',
        '[UPOS=NOUN]',
        '[UPOS=NUM]',
        '[UPOS=PRON]',
        '[UPOS=PROPN]',
        '[UPOS=PUNCT]',
        '[UPOS=SYM]',
        '[UPOS=VERB]',
        '[UPOS=VERB][SUBCAT=NEG]',
        '[UPOS=X]',
        '[VOICE=ACT]',
        '[VOICE=PSS]',
        '[WORD_ID=',
        '[NEWPARA=',
        '[BLACKLIST=',
        "[FOREIGN=FOREIGN]"
    }

    ## legacy multichars for long POS analyses
    old_poses = {
        '[POS=ADPOSITION]',
        '[POS=PUNCTUATION]',
        '[POS=PRONOUN]',
        '[POS=NUMERAL]',
        '[POS=SYMBOL]'
    }

    ## tags for analysing allomorphs
    allo_multichars = {
        '[ALLO=A]',
        '[ALLO=AN]',
        '[ALLO=EN]',
        '[ALLO=HAN]',
        '[ALLO=HEN]',
        '[ALLO=HIN]',
        '[ALLO=HON]',
        '[ALLO=HUN]',
        '[ALLO=HVN]',
        '[ALLO=HYN]',
        '[ALLO=HÄN]',
        '[ALLO=HÖN]',
        '[ALLO=IA]',
        '[ALLO=IDEN]',
        '[ALLO=IDEN]',
        '[ALLO=IEN]',
        '[ALLO=IHIN]',
        '[ALLO=IIN]',
        '[ALLO=IN]',
        '[ALLO=ISIIN]',
        '[ALLO=ITA]',
        '[ALLO=ITTEN]',
        '[ALLO=ITÄ]',
        '[ALLO=IÄ]',
        '[ALLO=JA]',
        '[ALLO=JÄ]',
        '[ALLO=JEN]',
        '[ALLO=NA]',
        '[ALLO=ON]',
        '[ALLO=SA]',
        '[ALLO=SEEN]',
        '[ALLO=TA]',
        '[ALLO=TEN]',
        '[ALLO=TÄ]',
        '[ALLO=UN]',
        '[ALLO=VN]',
        '[ALLO=YN]',
        '[ALLO=Ä]',
        '[ALLO=ÄN]',
        '[ALLO=ÖN]'
    }

    ## tags for analysing kotus dictionary classifications
    ktnkav_multichars = {
        '[KTN=1]', '[KTN=2]', '[KTN=3]', '[KTN=4]', '[KTN=5]',
        '[KTN=6]', '[KTN=7]', '[KTN=8]', '[KTN=9]', '[KTN=10]',
        '[KTN=11]', '[KTN=12]', '[KTN=13]', '[KTN=14]', '[KTN=15]',
        '[KTN=16]', '[KTN=17]', '[KTN=18]', '[KTN=19]', '[KTN=20]',
        '[KTN=21]', '[KTN=22]', '[KTN=23]', '[KTN=24]', '[KTN=25]',
        '[KTN=26]', '[KTN=27]', '[KTN=28]', '[KTN=29]', '[KTN=30]',
        '[KTN=31]', '[KTN=32]', '[KTN=33]', '[KTN=34]', '[KTN=35]',
        '[KTN=36]', '[KTN=37]', '[KTN=38]', '[KTN=39]', '[KTN=40]',
        '[KTN=41]', '[KTN=42]', '[KTN=43]', '[KTN=44]', '[KTN=45]',
        '[KTN=46]', '[KTN=47]', '[KTN=48]', '[KTN=49]', '[KTN=50]',
        '[KTN=51]', '[KTN=52]', '[KTN=53]', '[KTN=54]', '[KTN=55]',
        '[KTN=56]', '[KTN=57]', '[KTN=58]', '[KTN=59]', '[KTN=60]',
        '[KTN=61]', '[KTN=62]', '[KTN=63]', '[KTN=64]', '[KTN=65]',
        '[KTN=66]', '[KTN=67]', '[KTN=68]', '[KTN=69]', '[KTN=70]',
        '[KTN=71]', '[KTN=72]', '[KTN=73]', '[KTN=74]', '[KTN=75]',
        '[KTN=76]', '[KTN=77]', '[KTN=78]',
        '[KTN=1007]', '[KTN=1009]', '[KTN=1010]', '[KTN=1024]', '[KTN=1026]',
        '[KAV=A]', '[KAV=B]', '[KAV=C]', '[KAV=D]', '[KAV=E]',
        '[KAV=F]', '[KAV=G]', '[KAV=H]', '[KAV=I]', '[KAV=J]',
        '[KAV=K]', '[KAV=L]', '[KAV=M]',
        '[KAV=N]', '[KAV=O]', '[KAV=P]', '[KAV=T]'}

    ## mapping from omorfi data to omorfi analyses... Between the really really
    ## internal format and mildly internal format :-}
    stuff2omor = {
        ".sent": "[BOUNDARY=SENTENCE]",
        "Aa": "[ALLO=A]",
        "Aja": "[ALLO=JA]",
        "Ana": "[ALLO=NA]",
        "Asa": "[ALLO=SA]",
        "Aia": "[ALLO=IA]",
        "Ata": "[ALLO=TA]",
        "Atä": "[ALLO=TÄ]",
        "Aiä": "[ALLO=IÄ]",
        "Aita": "[ALLO=ITA]",
        "Aitä": "[ALLO=ITÄ]",
        "Aitten": "[ALLO=ITTEN]",
        "Aiden": "[ALLO=IDEN]",
        "Aiin": "[ALLO=IIN]",
        "Aihin": "[ALLO=IHIN]",
        "Aseen": "[ALLO=SEEN]",
        "Aisiin": "[ALLO=ISIIN]",
        "Aien": "[ALLO=IEN]",
        "Ajen": "[ALLO=JEN]",
        "Aan": "[ALLO=AN]",
        "Aen": "[ALLO=EN]",
        "Ain": "[ALLO=IN]",
        "Aon": "[ALLO=ON]",
        "Aun": "[ALLO=UN]",
        "Ayn": "[ALLO=YN]",
        "Aän": "[ALLO=ÄN]",
        "Aön": "[ALLO=ÖN]",
        "Ahan": "[ALLO=HAN]",
        "Ahen": "[ALLO=HEN]",
        "Ahin": "[ALLO=HIN]",
        "Ahon": "[ALLO=HON]",
        "Ahun": "[ALLO=HUN]",
        "Ahyn": "[ALLO=HYN]",
        "Ahän": "[ALLO=HÄN]",
        "Ahön": "[ALLO=HÖN]",
        "Aten": "[ALLO=TEN]",
        "Ajä": "[ALLO=JÄ]",
        "Aä": "[ALLO=Ä]",
        "Bc": "[BOUNDARY=COMPOUND]",
        "B-": "[COMPOUND_FORM=OMIT]",
        "Bxxx": "[COMPOUND_FORM=MISSING-]",
        "B→": "[POSITION=SUFFIX]",
        "B←": "[POSITION=PREFIX]",
        "Cma": "[PCP=AGENT]",
        "Cmaton": "[PCP=NEG]",
        "Cva": "[PCP=VA]",
        "Cnut": "[PCP=NUT]",
        "Cpos": "[CMP=POS]",
        "Ccmp": "[CMP=CMP]",
        "Csup": "[CMP=SUP]",
        "Dhko": "[DRV=HKO]",
        "Dinen": "[DRV=INEN]",
        "Dnen": "[DRV=NEN]",
        "Dla": "[DRV=LA]",
        "Dlainen": "[DRV=LAINEN]",
        "Dllinen": "[DRV=LLINEN]",
        "Disa": "[DRV=ISA]",
        "Dja": "[DRV=JA]",
        "Dmaisilla": "[DRV=MAISILLA]",
        "Dminen": "[DRV=MINEN]",
        "Dmainen": "[DRV=MAINEN]",
        "Dtu": "[DRV=TU]",
        "Dnut": "[DRV=NUT]",
        "Dva": "[DRV=VA]",
        "Dtava": "[DRV=TAVA]",
        "Dma": "[DRV=MA]",
        "Dmaton": "[DRV=MATON]",
        "Ds": "[DRV=S]",
        "Dsti": "[DRV=STI]",
        "Dttaa": "[DRV=TTAA]",
        "Dtattaa": "[DRV=TATTAA]",
        "Dtatuttaa": "[DRV=TATUTTAA]",
        "Dtuttaa": "[DRV=TUTTAA]",
        "Dttain": "[DRV=TTAIN]",
        "Du": "[DRV=U]",
        "Duus": "[DRV=UUS]",
        "Dmpi": "[DRV=MPI]",
        "Dtar": "[DRV=TAR]",
        "Dton": "[DRV=TON]",
        "Din": "[DRV=IN]",
        "Din²": "[DRV=IN²]",
        "Ia": "[INF=A]",
        "Ie": "[INF=E]",
        "Ima": "[INF=MA]",
        "Iminen": "[INF=MINEN]",
        "Ncon": "[NEG=CON]",
        "Nneg": "[SUBCAT=NEG]",
        "Npl": "[NUM=PL]",
        "Nsg": "[NUM=SG]",
        "N??": "",
        "Osg1": "[POSS=SG1]",
        "Osg2": "[POSS=SG2]",
        "O3": "[POSS=3]",
        "Opl1": "[POSS=PL1]",
        "Opl2": "[POSS=PL2]",
        "Ppl1": "[PERS=PL1]",
        "Ppl2": "[PERS=PL2]",
        "Ppl3": "[PERS=PL3]",
        "Psg0": "[PERS=SG0]",
        "Psg1": "[PERS=SG1]",
        "Psg2": "[PERS=SG2]",
        "Psg3": "[PERS=SG3]",
        "Ppe4": "[PERS=PE4]",
        "Qka": "[CLIT=KA]",
        "Qs": "[CLIT=S]",
        "Qpa": "[CLIT=PA]",
        "Qko": "[CLIT=KO]",
        "Qkin": "[CLIT=KIN]",
        "Qkaan": "[CLIT=KAAN]",
        "Qhan": "[CLIT=HAN]",
        "Tcond": "[MOOD=COND]",
        "Timp": "[MOOD=IMPV]",
        "Tpast": "[MOOD=INDV][TENSE=PAST]",
        "Tpot": "[MOOD=POTN]",
        "Topt": "[MOOD=OPT]",
        "Tpres": "[MOOD=INDV][TENSE=PRESENT]",
        "Uarch": "[STYLE=ARCHAIC]",
        "Udial": "[STYLE=DIALECTAL]",
        "Unonstd": "[STYLE=NONSTANDARD]",
        "Urare": "[STYLE=RARE]",
        "Vact": "[VOICE=ACT]",
        "Vpss": "[VOICE=PSS]",
        "Xabe": "[CASE=ABE]",
        "Xabl": "[CASE=ABL]",
        "Xade": "[CASE=ADE]",
        "Xall": "[CASE=ALL]",
        "Xcom": "[CASE=COM]",
        "Xela": "[CASE=ELA]",
        "Xess": "[CASE=ESS]",
        "Xgen": "[CASE=GEN]",
        "Xill": "[CASE=ILL]",
        "Xine": "[CASE=INE]",
        "Xins": "[CASE=INS]",
        "Xnom": "[CASE=NOM]",
        "Xpar": "[CASE=PAR]",
        "Xtra": "[CASE=TRA]",
        "Xlat": "[CASE=LAT]",
        "Xacc": "[CASE=ACC]",
        "X???": "",
        "AUX": "[UPOS=AUX]",
        "DET": "[UPOS=DET]",
        "NOUN": "[UPOS=NOUN]",
        "VERB": "[UPOS=VERB]",
        "ADV": "[UPOS=ADV]",
        "ADP": "[UPOS=ADP]",
        "ADJ": "[UPOS=ADJ]",
        "INTJ": "[UPOS=INTJ]",
        "CCONJ": "[UPOS=CCONJ]",
        "SCONJ": "[UPOS=SCONJ]",
        "PRON": "[UPOS=PRON]",
        "SYM": "[UPOS=SYM]",
        "NUM": "[UPOS=NUM]",
        "PROPN": "[UPOS=PROPN]",
        "X": "[UPOS=X]",
        "PUNCT": "[UPOS=PUNCT]",
        "ABESSIVE": "[LEX=ABE]",
        "ABLATIVE": "[LEX=ABL]",
        "ADESSIVE": "[LEX=ADE]",
        "ALLATIVE": "[LEX=ALL]",
        "ELATIVE": "[LEX=ELA]",
        "LOCATIVE": "[LEX=LOC]",
        "GENITIVE": "[LEX=GEN]",
        "ILLATIVE": "[LEX=ILL]",
        "INESSIVE": "[LEX=INE]",
        "INSTRUCTIVE": "[LEX=INS]",
        "PARTITIVE": "[LEX=PAR]",
        "SEPARATIVE": "[LEX=SEP]",
        "LATIVE": "[LEX=LAT]",
        "DERSTI": "[LEX=STI]",
        "DERTTAIN": "[LEX=TTAIN]",
        "CONJUNCTION": "",
        "COORDINATING": "[UPOS=CCONJ]",
        "COMPARATIVE": "[UPOS=SCONJ][CONJ=COMPARATIVE]",
        "PRONOUN": "[POS=PRONOUN]",
        "ADVERBIAL": "[UPOS=SCONJ][CONJ=ADVERBIAL]",
        "NUMERAL": "[POS=NUMERAL]",
        "CARDINAL": "[NUMTYPE=CARD]",
        "ORDINAL": "[NUMTYPE=ORD]",
        "ORD": "[NUMTYPE=ORD]",
        "DIGIT": "[SUBCAT=DIGIT]",
        "DECIMAL": "[SUBCAT=DECIMAL]",
        "ROMAN": "[SUBCAT=ROMAN]",
        "QUALIFIER": "[SUBCAT=QUALIFIER]",
        "ACRONYM": "[ABBR=ACRONYM]",
        "ABBREVIATION": "[ABBR=ABBREVIATION]",
        "SUFFIX": "[SUBCAT=SUFFIX]",
        "PREFIX": "[SUBCAT=PREFIX]",
        "INTERJECTION": "[SUBCAT=INTERJECTION]",
        "ADPOSITION": "[POS=ADPOSITION]",
        "DEMONSTRATIVE": "[PRONTYPE=DEM]",
        "QUANTOR": "[SUBCAT=QUANTIFIER]",
        "QUANTIFIER": "[SUBCAT=QUANTIFIER]",
        "PERSONAL": "[PRONTYPE=PRS]",
        "INDEFINITE": "[PRONTYPE=IND]",
        "INTERROGATIVE": "[PRONTYPE=INT]",
        "REFLEXIVE": "[SUBCAT=REFLEXIVE]",
        "RELATIVE": "[PRONTYPE=REL]",
        "RECIPROCAL": "[PRONTYPE=REC]",
        "PL1": "[PERS=PL1]",
        "PL2": "[PERS=PL2]",
        "PL3": "[PERS=PL3]",
        "SG0": "[PERS=SG0]",
        "SG1": "[PERS=SG1]",
        "SG2": "[PERS=SG2]",
        "SG3": "[PERS=SG3]",
        "PE4": "[PERS=PE4]",
        "COMP": "[CMP=CMP]",
        "SUPERL": "[CMP=SUP]",
        "ARCHAIC": "[STYLE=ARCHAIC]",
        "DIALECTAL": "[STYLE=DIALECTAL]",
        "NONSTANDARD": "[STYLE=NONSTANDARD]",
        "RARE": "[STYLE=RARE]",
        "TITLE": "[SEM=TITLE]",
        "TIME": "[SEM=TIME]",
        "CURRENCY": "[SEM=CURRENCY]",
        "MEDIA": "[SEM=MEDIA]",
        "POLIT": "[SEM=POLIT]",
        "MEASURE": "[SEM=MEASURE]",
        "MALE": "[SEM=MALE]",
        "FEMALE": "[SEM=FEMALE]",
        "CULTGRP": "[PROPER=CULTGRP]",
        "PRODUCT": "[PROPER=PRODUCT]",
        "ARTWORK": "[PROPER=ARTWORK]",
        "EVENT": "[PROPER=EVENT]",
        "FIRST": "[PROPER=FIRST]",
        "LAST": "[PROPER=LAST]",
        "GEO": "[PROPER=GEO]",
        "ORG": "[PROPER=ORG]",
        "MISC": "[PROPER=MISC]",
        "COUNTRY": "[SEM=COUNTRY]",
        "INHABITANT": "[SEM=INHABITANT]",
        "LANGUAGE": "[SEM=LANGUAGE]",
        "PUNCTUATION": "[POS=PUNCTUATION]",
        "DASH": "[SUBCAT=DASH]",
        "SPACE": "[SUBCAT=SPACE]",
        "COMMA": "[SUBCAT=COMMA]",
        "ARROW": "[SUBCAT=ARROW]",
        "PREPOSITION": "[ADPTYPE=PREP]",
        "POSTPOSITION": "[ADPTYPE=POST]",
        "MULTIPLICATIVE": "[NUMTYPE=MULT]",
        "FRACTION": "[NUMTYPE=FRAC]",
        "CLAUSE-BOUNDARY": "[BOUNDARY=CLAUSE]",
        "SENTENCE-BOUNDARY": "[BOUNDARY=SENTENCE]",
        "INITIAL-QUOTE": "[SUBCAT=QUOTATION][POSITION=INITIAL]",
        "FINAL-QUOTE": "[SUBCAT=QUOTATION][POSITION=FINAL]",
        "INITIAL-BRACKET": "[SUBCAT=BRACKET][POSITION=INITIAL]",
        "FINAL-BRACKET": "[SUBCAT=BRACKET][POSITION=FINAL]",
        "UNSPECIFIED": "",
        "FTB3man": "",
        "LEMMA-START": "[WORD_ID=",
        "LEMMA-END": "]",
        "CCONJ|VERB": "[UPOS=VERB][SUBCAT=NEG]",
        "FTB3MAN": "",
        "XForeign": "[FOREIGN=FOREIGN]",
        "BLACKLISTED": "[BLACKLIST=",
        "": ""}

    def __init__(self, verbose=False, **kwargs):
        """Create a conversion with given verbosity and options

        @param verbose set to true ot print output while processing
        @param sem  set to true to include semantic tags
        @param allo  set to true to include allomiorph tags
        @param props  set to true to include NER tags
        @param ktnkav  set to true to get KOTUS dictionary paradigms tags
        @param newparas  set to true to get omorfi paradigms tags
        """
        for stuff, omor in self.stuff2omor.items():
            if len(omor) < 2:
                continue
            elif omor not in self.common_multichars | self.old_poses | \
                    self.allo_multichars:
                just_fail(
                    "There are conflicting formattings in here!\n" +
                    omor + " corresponding " + stuff +
                    " is not a valid defined omor multichar_symbol!")
        ## verbosity
        self.verbose = verbose
        ## if semantics tags enabled
        self.semantics = True
        if 'sem' not in kwargs or not kwargs['sem']:
            for k, v in self.stuff2omor.items():
                if "SEM=" in v:
                    self.stuff2omor[k] = ""
            self.semantics = False
        ## if allomorph tags enabled
        self.allo = True
        if 'allo' not in kwargs or not kwargs['allo']:
            for k, v in self.stuff2omor.items():
                if "ALLO=" in v:
                    self.stuff2omor[k] = ""
            self.allo = False
        ## if NER tags enabled
        self.props = True
        if 'props' not in kwargs or not kwargs['props']:
            for k, v in self.stuff2omor.items():
                if "PROPER=" in v:
                    self.stuff2omor[k] = ""
            self.props = False
        ## if kotus paradimgs tags enabled
        self.ktnkav = True
        if 'ktnkav' not in kwargs or not kwargs['ktnkav']:
            for k, v in self.stuff2omor.items():
                if "KTN=" in v or "KAV=" in v:
                    self.stuff2omor[k] = ""
            self.ktnkav = False
        ## if omorfi paradigms tags enabled
        self.newparas = True
        if 'newparas' not in kwargs or not kwargs['newparas']:
            for k, v in self.stuff2omor.items():
                if "NEWPARA=" in v:
                    self.stuff2omor[k] = ""
            self.newparas = False

    def stuff2lexc(self, stuff):
        """Convert omorfi internal tag to omorfi lexc analysis tag

        @return lexc-formatted omorfi analysis
        """
        if stuff == '0':
            return "0"
        if stuff in self.stuff2omor:
            return lexc_escape(self.stuff2omor[stuff])
        else:
            fail_formatting_missing_for(stuff, "omor")
            return "ERRORMACRO"

    def analyses2lexc(self, anals, surf):
        """Convert omorfi analysis tags string into omorfi lexc tags.

        @return lexc-formatted tag string with potentially stem fragments etc.
        """
        omorstring = ''
        for tag in anals.split('|'):
            if tag == '@@COPY-STEM@@':
                omorstring += lexc_escape(surf)
            elif tag.startswith('@@LITERAL') and tag.endswith('@@'):
                omorstring += lexc_escape(tag[len('@@LITERAL'):-len('@@')])
            else:
                omorstring += self.stuff2lexc(tag)
        return omorstring

    def continuation2lexc(self, anals, surf, cont):
        """Create lexc entry from omorfi continuation data

        @return lexc-format continuation entry
        """
        tags = self.analyses2lexc(anals, surf)
        surf = lexc_escape(surf)
        return "%s:%s\t%s ;\n" % (tags, surf, cont)

    def guesser2lexc(self, regex, deletion, cont):
        """Create a lexc guesser entry from omorfi continuation data

        @return lexc-format guesser lexical entry
        """
        if not regex:
            regex = ''
        regex = egrep2xerox(regex)
        regex = regex_delete_surface(regex, deletion)
        return "< ?* %s >\t%s ;\n" % (regex, cont)

    def wordmap2lexc(self, wordmap):
        '''
        format string for canonical omor format for morphological analysis

        @return lexc-formatted lexical entry for omorfi analyses
        '''
        if wordmap['stub'] == ' ':
            # do not include normal white space for now
            return ""
        wordmap['stub'] = lexc_escape(wordmap['stub'])
        if int(wordmap['homonym']) == 1:
            wordmap['analysis'] = "[WORD_ID=%s]" % (
                lexc_escape(wordmap['lemma']))
        else:
            wordmap['analysis'] = "[WORD_ID=%s_%s]" % (
                lexc_escape(wordmap['lemma']), wordmap['homonym'])
        if wordmap['numtype'] and wordmap['numtype'] == 'ORD':
            wordmap['analysis'] += self.stuff2lexc('ADJ')
        else:
            wordmap['analysis'] += self.stuff2lexc(wordmap['upos'])
        if wordmap['is_suffix']:
            wordmap['analysis'] += self.stuff2lexc('SUFFIX')
        if wordmap['is_prefix']:
            wordmap['analysis'] += self.stuff2lexc('PREFIX')
            if wordmap['upos'] == 'ADJ':
                wordmap['analysis'] += self.stuff2lexc('Cpos')

        if wordmap['blacklist']:
            wordmap['analysis'] += self.stuff2lexc('BLACKLISTED') + \
                wordmap['blacklist'] + ']'
        if wordmap['particle']:
            for pclass in wordmap['particle'].split('|'):
                wordmap['analysis'] += self.stuff2lexc(pclass)

        if wordmap['symbol']:
            for subcat in wordmap['symbol'].split('|'):
                wordmap['analysis'] += self.stuff2lexc(subcat)

        if wordmap['prontype']:
            for stuff in wordmap['prontype'].split("|"):
                wordmap['analysis'] += self.stuff2lexc(stuff)
        if wordmap['lex']:
            for stuff in wordmap['lex'].split("|"):
                wordmap['analysis'] += self.stuff2lexc(stuff)
        if wordmap['abbr']:
            for stuff in wordmap['abbr'].split("|"):
                wordmap['analysis'] += self.stuff2lexc(stuff)
        if wordmap['numtype']:
            for stuff in wordmap['numtype'].split("|"):
                wordmap['analysis'] += self.stuff2lexc(stuff)
        if wordmap['adptype']:
            for stuff in wordmap['adptype'].split("|"):
                wordmap['analysis'] += self.stuff2lexc(stuff)

        if self.props and wordmap['proper_noun_class']:
            wordmap['analysis'] += \
                self.stuff2lexc(wordmap['proper_noun_class'])
        if self.semantics and wordmap['sem']:
            wordmap['analysis'] += self.stuff2lexc(wordmap['sem'])

        if wordmap['style']:
            wordmap['analysis'] += self.stuff2lexc(wordmap['style'])

        if self.ktnkav and wordmap['upos'] != 'ACRONYM':
            tag = "[KTN=%s]" % (lexc_escape(wordmap['kotus_tn']))
            if tag in self.ktnkav_multichars:
                wordmap['analysis'] += tag
                if wordmap['kotus_av']:
                    wordmap['analysis'] += "[KAV=%(kotus_av)s]" % (wordmap)
        if self.newparas:
            wordmap['analysis'] += "[NEWPARA=%s]" % (wordmap['new_para'],)

        # match WORD_ID= with epsilon, then stub and lemma might match
        lex_stub = '0' + wordmap['stub']

        lexc_line = "%s:%s\t%s\t;" % (wordmap['analysis'], lex_stub,
                                      wordmap['new_para'])
        if 'BLACKLISTED' in wordmap['new_para']:
            return "! ! !" + lexc_line
        else:
            return lexc_line

    def multichars_lexc(self):
        """Create analysis tags description for lexc for omorfi tags

        @return lexc-formatted Multichar_symbols block for omorfi analyses
        """
        multichars = "Multichar_Symbols\n"
        multichars += "!! OMOR multichars:\n"
        for mcs in self.common_multichars:
            multichars += mcs + "\n"
        multichars += Formatter.multichars_lexc(self)
        return multichars

    def root_lexicon_lexc(self):
        """Create omorfi root lexicon.

        @return elxc-formatted Root lexicon string for omorfi
        """
        root = Formatter.root_lexicon_lexc(self)
        if True:
            # want co-ordinated hyphens
            root += "!! LEXICONS that can be co-ordinated hyphen -compounds\n"
            root += self.stuff2lexc('B→') + ':-   NOUN ;\n'
            root += self.stuff2lexc('B→') + ':-   ADJ ;\n'
            root += self.stuff2lexc('B→') + ':-   SUFFIX ;\n'
        if False:
            root += "0   TAGGER_HACKS    ;\n"
        return root

    @staticmethod
    def get_lemmas(s, hacks=None):
        '''Get lemma(s) from analysed token.'''
        re_lemma = re.compile(r"\[WORD_ID=([^]]*)\]")
        escanal = s.replace('[WORD_ID=]]',
                            '[WORD_ID=@RIGHTSQUAREBRACKET@]')
        lemmas = re_lemma.finditer(escanal)
        rv = []
        for lemma in lemmas:
            s = lemma.group(1)
            for i in range(32):
                hnsuf = '_' + str(i)
                if s.endswith(hnsuf):
                    s = s[:-len(hnsuf)]
            if s == '@RIGHTSQUAREBRACKET@':
                s = ']'
            rv += [s]
        # legacy pron hack
        if len(rv) == 1 and rv[0] in ['me', 'te', 'he', 'nämä', 'ne'] and\
                OmorFormatter.get_upos(s) == 'PRON' and hacks:
            if rv[0] == 'me':
                rv[0] = 'minä'
            elif rv[0] == 'te':
                rv[0] = 'sinä'
            elif rv[0] == 'he':
                rv[0] = 'hän'
            elif rv[0] == 'nämä':
                rv[0] = 'tämä'
            elif rv[0] == 'ne':
                rv[0] = 'se'
        return rv

    @staticmethod
    def get_last_feat(s, feat):
        '''Get last (effective) value for the given morphological feature.

        This function tries to determine the most likely morphosyntactic
        feature values from complex analyses, e.g. with compounds and
        derivations the most relevant ones for the whole token.
        '''
        re_feat = re.compile(r"\[" + feat + r"=([^]]*)\]")
        feats = re_feat.finditer(s)
        rv = ""
        for f in feats:
            rv = f.group(1)
        return rv

    @staticmethod
    def get_last_feats(s):
        '''Get last (effective) value for the given morphological feature.

        This function tries to determine the most likely morphosyntactic
        feature values from complex analyses, e.g. with compounds and
        derivations the most relevant ones for the whole token.
        '''
        re_feats = re.compile(r"\[[A-Z_]*=[^]]*\]")
        rvs = list()
        feats = re_feats.finditer(s)
        for feat in feats:
            if 'WORD_ID=' in feat.group(0):
                # feats reset on word boundary
                rvs = list()
            else:
                rvs.append(feat.group(0))
        return rvs

    @staticmethod
    def get_upos(s, deriv_munging=True):
        '''Get Universal Part-of-Speech.'''
        upos = OmorFormatter.get_last_feat(s, "UPOS")
        if deriv_munging:
            drv = OmorFormatter.get_last_feat(s, "DRV")
            if upos == 'VERB' and drv == 'MINEN':
                upos = 'NOUN'
        return upos


# self test
if __name__ == '__main__':
    formatter = OmorFormatter()
    exit(0)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Functions to format lexc from omorfi data."""

# Author: Omorfi contributors <omorfi-devel@groups.google.com> 2015

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

# functions for formatting the database data to lexc

from .settings import deriv_boundary, morph_boundary, newword_boundary, optional_hyphen, stub_boundary, word_boundary
from .string_manglers import lexc_escape


def format_copyright_lexc():
    return """
! This automatically generated lexc data is originated from
! omorfi database.
! Copyright (c) 2014 Omorfi contributors

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, version 3 of the License

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


# generics
def format_continuation_lexc_generic(anals, surf, cont):
    surf = lexc_escape(surf)
    return "%s:%s\t%s ; \n" % (surf.replace(optional_hyphen, newword_boundary),
                               surf, cont)


def format_wordmap_lexc_generic(wordmap):
    wordmap['analysis'] = lexc_escape(wordmap['stub']) + stub_boundary
    retvals = []
    lex_stub = lexc_escape(wordmap['stub'])
    retvals += ["%s:%s\t%s\t;" %
                (wordmap['analysis'], lex_stub, wordmap['new_para'])]
    return "\n".join(retvals)


def format_wordmap_lexc_labeled_segments(wordmap):
    wordmap['analysis'] = lexc_escape(
        wordmap['stub']) + "[UPOS=" + wordmap['upos'] + ']'
    wordmap['analysis'] = wordmap['analysis'].replace(word_boundary, "[WB=+]")
    wordmap['analysis'] = wordmap['analysis'].replace(
        newword_boundary, "[WB=?]")
    retvals = []
    lex_stub = lexc_escape(wordmap['stub'])
    retvals += ["%s:%s\t%s\t;" %
                (wordmap['analysis'], lex_stub, wordmap['new_para'])]
    return "\n".join(retvals)


def format_continuation_lexc_labeled_segments(anals, surf, cont):
    surf = lexc_escape(surf)
    # mostly suffixes: e.g.
    # >i>ssa Pl|Ine -> |i|PL|ssa|INE
    # >i -> |i|ACT|PAST|SG3
    foo = surf
    foo = foo.replace(morph_boundary, "[MB=LEFT]", 1)
    foo = foo.replace(newword_boundary, "[WB=?]")
    foo = foo.replace(word_boundary, "[WB=+]")
    restanals = []
    for anal in anals.split('|'):
        if anal.startswith("D") and "{DB}" in foo:
            foo = foo.replace(deriv_boundary, "[DB=" + anal + "]", 1)
        elif "{MB}" in foo:
            foo = foo.replace(morph_boundary, "[MB=" + anal + "]", 1)
        else:
            restanals.append(anal)
    if restanals and len(restanals) > 0:
        foo += "[TRAILS=→" + "][?=".join(restanals) + "]"

    return "%s:%s\t%s ; \n" % (foo.replace(optional_hyphen, newword_boundary),
                               surf, cont)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Functions to format apertium monodix from omorfi data."""

# Author: Omorfi contributors <omorfi-devel@groups.google.com> 2015

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# utils to format apertium style data from omorfi database values

from .apertium_formatter import ApertiumFormatter
from ..error_logging import fail_formatting_missing_for
from ..settings import (word_boundary, morph_boundary, deriv_boundary,
                        weak_boundary, stub_boundary, optional_hyphen)

monodix_sdefs = {
    'abbr',
    'abe',
    'abl',
    'acc',
    'actv',
    'ade',
    'adj',
    'adv',
    'agent',
    'all',
    'ant',
    'card',
    'cmp',
    'cnjcoo',
    'cnjsub',
    'com',
    'comp',
    'cond',
    'conneg',
    'dem',
    'ela',
    'enc',
    'ess',
    'f',
    'gen',
    'ij',
    'ill',
    'imp',
    'impers',
    'ind',
    'indv',
    'ine',
    'infa',
    'infe',
    'infma',
    'infminen',
    'ins',
    'itg',
    'lat',
    'm',
    'mf',
    'n',
    'neg',
    'nom',
    'np',
    'nt',
    'num',
    'ord',
    'p1',
    'p2',
    'p3',
    'par',
    'part',
    'past',
    'pasv',
    'pl',
    'pneg',
    'pos',
    'post',
    'pot',
    'pp',
    'pprs',
    'pr',
    'pri',
    'prn',
    'pxpl1',
    'pxpl2',
    'pxsg1',
    'pxsg2',
    'pxsp3',
    'qst',
    'rec',
    'reflex',
    'rel',
    'sg',
    'sup',
    'top',
    'tra',
    'vblex',
    'ND', 'ERROR'}

stuff2monodix = {
    "Aiden": "",
    "Aien": "",
    "Aiin": "",
    "Ain": "",
    "Ayn": "",
    "Aän": "",
    "Aön": "",
    "Aisiin": "",
    "Aseen": "",
    "Aä": "",
    "Ajä": "",
    "Atä": "",
    "Ajen": "",
    "Aten": "",
    "Ahin": "",
    "Ahen": "",
    "Ahyn": "",
    "Aihin": "",
    "Aiä": "",
    "Ana": "",
    "Asa": "",
    "Aitten": "",
    "Aan": "",
    "Aen": "",
    "Ahan": "",
    "Ahon": "",
    "Ahun": "",
    "Aon": "",
    "Aun": "",
    "Aa": "",
    "Aia": "",
    "Aita": "",
    "Aja": "",
    "Ahän": "",
    "Ahön": "",
    "Aitä": "",
    "Ata": "",
    "ABBREVIATION": "abbr",
    "ACRONYM": "abbr",
    "ADJECTIVE": "adj",
    "ADJ": "adj",
    "ADPOSITION": "post",
    "ADVERB": "adv",
    "ADVERBIAL": "cnjadv",
    "AINF_arg": "vaux",
    "ARTWORK": "",
    "Bxxx": "-",
    "B-": "-",
    "B←": "-",
    "B→": "-",
    "Bc": "+",
    "CARDINAL": "card",
    "Ccmp": "com",
    "CLAUSE-BOUNDARY": "",
    "Cma": "agent",
    "Cmaton": "pneg",
    "Cnut": "pp",
    "COMPARATIVE": "cnjsub",
    "COMP": "com",
    "CONJUNCTION": "",
    "CONJUNCTIONVERB": "cnjcoo><vblex",
    "COORDINATING": "cnjcoo",
    "Cpos": "pos",
    "Csup": "sup",
    "CULTGRP": "",
    "Cva": "pprs",
    "DASH": "guio",
    "DECIMAL": "",
    "DEMONSTRATIVE": "dem",
    "DIGIT": "",
    "Dinen": "+inen<adj",
    "Din": "+in<n",
    "Din²": "+in<adj",
    "Dja": "+ja<n",
    "Dla": "+la<n",
    "Dmaisilla": "+maisilla<adv",
    "Dma": "+ma<n",
    "Dmaton": "+maton<adj",
    "Dminen": "+minen<n",
    "Dmainen": "+mainen<adj",
    "Dnen": "+nen<n",
    "Dllinen": "+llinen<n",
    "Dmpi": "+mpi<adj",
    "Dnut": "+nut<adj",
    "Dhko": "+hko<adj",
    "Disa": "+isa<adj",
    "Dtar": "+tar<n",
    "Dlainen": "+lainen<n",
    "Dton": "+ton<adj",
    "Dtuttaa": "+tuttaa<vblex",
    "Ds": "+s<n",
    "Dsti": "+sti<adv",
    "Dtattaa": "+tattaa<vblex",
    "Dtatuttaa": "+tatuttaa<vblex",
    "Dtava": "+tava<adj",
    "Dttaa": "+ttaa<vblex",
    "Dttain": "+ttain<adv",
    "Dtu": "+tu<adj",
    "Du": "+u<n",
    "Duus": "+uus<n",
    "Dva": "+va<adj",
    "EVENT": "",
    "FINAL-BRACKET": "rpar",
    "FINAL-QUOTE": "rquot",
    "FIRST": "ant",
    "GEO": "top",
    "Ia": "infa",
    "Ie": "infe",
    "Ima": "infma",
    "Iminen": "infminen",
    "INDEFINITE": "ind",
    "INITIAL-BRACKET": "lpar",
    "INITIAL-QUOTE": "lquot",
    "INTRANSITIVE_arg": "vblex",
    "INTERJECTION": "ij",
    "INTERROGATIVE": "itg",
    "LAST": "ant",
    "LEMMA-START": "",
    "LEMMA-END": "",
    "MAINF_arg": "vaux",
    "MEDIA": "",
    "MISC": "",
    "Ncon": "conneg",
    "N??": "ND",
    "Nneg": "neg",
    "NOUN": "n",
    "Npl": "pl",
    "Nsg": "sg",
    "NUM": "num",
    "NUMERAL": "num",
    "O3": "pxsp3",
    "Opl1": "pxpl1",
    "Opl2": "pxpl2",
    "ORDINAL": "ord",
    "ORD": "ord",
    "ORG": "",
    "Osg1": "pxsg1",
    "Osg2": "pxsg2",
    "PARTICLE": "part",
    "PERSONAL": "pers",
    "PL1": "p1",
    "PL2": "p2",
    "PL3": "p3",
    "Ppe4": "impers",
    "Ppl1": "p1><pl",
    "Ppl2": "p2><pl",
    "Ppl3": "p3><pl",
    "PRONOUN": "prn",
    "PRODUCT": "",
    "PROPER": "np",
    "PROPN": "np",
    "Psg0": "sg",
    "Psg1": "p1><sg",
    "Psg2": "p2><sg",
    "Psg3": "p3><sg",
    "PUNCTUATION": "",
    "Qhan": "+han<enc",
    "Qkaan": "+kaan<enc",
    "Qka": "+ka<enc",
    "Qkin": "+kin<enc",
    "Qko": "+ko<qst",
    "Qpa": "+pa<enc",
    "Qs": "+s<enc",
    "QUALIFIER": "adj",
    "QUANTOR": "qu",
    "RECIPROCAL": "rec",
    "REFLEXIVE": "reflex",
    "RELATIVE": "rel",
    "ROMAN": "",
    ".sent": "",
    "SG1": "p1",
    "SG2": "p2",
    "SG3": "p3",
    "SENTENCE-BOUNDARY": "sent",
    "SPACE": "",
    "SUPERL": "sup",
    "Tcond": "cond",
    "Timp": "imp",
    "Topt": "",
    "Tpast": "past",
    "Tpot": "pot",
    "Tpres": "pri",
    "Uarch": "",
    "Udial": "",
    "Unonstd": "",
    "UNSPECIFIED": "part",
    "Urare": "",
    "Vact": "actv",
    "VERB": "vblex",
    "Vpss": "pasv",
    "X???": "",
    "Xabe": "abe",
    "Xabl": "abl",
    "Xacc": "acc",
    "Xade": "ade",
    "Xall": "all",
    "Xcom": "com",
    "Xela": "ela",
    "Xess": "ess",
    "Xgen": "gen",
    "Xill": "ill",
    "Xine": "ine",
    "Xins": "ins",
    "Xlat": "lat",
    "Xnom": "nom",
    "Xpar": "par",
    "Xtra": "tra",
    "FTB3man": "",
    "X": "",
    "XForeign": "",
    ".": "",
    "": ""
}


def format_monodix_licence():
    return """<!--
  Copyright (c) 2014 Omorfi contributors

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, version 3 of the License

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
-->"""


def format_monodix_alphabet():
    """Finnish alphabet as in CLDR 24"""
    return ("<alphabet>abcdefghijklmnopqrsštuvwxyzžåäö"
            "ABCDEFGHIJKLMNOPQRSŠTUVWXYZŽ"
            "áàâãčçđéèêëǧǥȟíîïǩńñŋôõœřßŧúùûÿüʒǯæø"
            "ÁÀÂÃČÇÐÉÈÊËǦÍÎÏĸŃÑŊÔÕŘÚÙÛŸÜƷǮÆØ"
            "</alphabet>")


def format_monodix_sdefs():
    sdefs = '  <sdefs>\n'
    for sdef in monodix_sdefs:
        sdefs += '    <sdef n="' + sdef + '"/>\n'
    sdefs += '  </sdefs>\n'
    return sdefs


def format_monodix_l(s):
    if s != '0':
        return s.replace(' ', '<b/>').replace(word_boundary, '') \
            .replace(morph_boundary, '').replace(deriv_boundary, '')\
            .replace(stub_boundary, '').replace(optional_hyphen, '')\
            .replace(weak_boundary, '')
    else:
        return ''


def format_monodix_r(anals, stem):
    r = ''
    if anals != '0':
        for anal in anals.split('|'):
            if anal == '@@COPY-STEM@@':
                r += stem
            elif anal.startswith('@@LITERAL') and anal.endswith('@@'):
                r += anal[len('@@LITERAL'):-len('@@')]
            else:
                r += format_monodix_s(anal)
    return r


def format_monodix_s(stuff):
    s = ''
    if stuff in stuff2monodix:
        s += '<s n="' + stuff2monodix[stuff] + '"/>'
    else:
        fail_formatting_missing_for(stuff, "monodix")
        s = '<s n="ERROR"/>'
    if '><' in s:
        s = s.replace('><', '"/><s n="')
    elif '<s n="+' in s:
        s = s.replace('<s n="', '').replace('<', '<s n="')
    elif '"+"' in s:
        s = '+'
    elif '""' in s:
        s = ''
    elif '"-"' in s:
        s = '-'
    return s


def format_monodix_par(cont):
    if cont.startswith("X_FORGN"):
        return ''
    return '<par n="' + cont.lower().replace('_', '__') + '"/>'


def format_monodix_pardef(fields):
    pardef = ''
    for cont in fields[3:]:
        pardef += '      <e>'
        if fields[1] == fields[2]:
            pardef += '<i>' + format_monodix_l(fields[2]) + '</i>'
        else:
            pardef += '<p><l>' + format_monodix_l(fields[2]) + '</l>'
            pardef += '<r>' + format_monodix_r(fields[1], fields[2]) + \
                      '</r></p>'
        if cont != '#':
            pardef += format_monodix_par(cont)
        pardef += '</e>\n'
    return pardef


def format_monodix_entry(wordmap):
    if wordmap['new_para'] == 'X_IGNORE' or wordmap['lemma'] == ' ':
        return ''
    e = '<e lm="' + wordmap['lemma'].replace('&', '&amp;')\
        .replace('"', '&quot;').replace("<", "&lt;") + '">'
    e += '<p><l>' + \
        wordmap['stub'].replace(word_boundary, '').replace(
            '&', '&amp;').replace('<', '&lt;') + '</l>'
    e += '<r>'
    e += wordmap['lemma'].replace('&', '&amp;').replace("<", "&lt;")
    e += format_monodix_s(wordmap['real_pos'] or wordmap['pos'])
    e += '</r></p>'
    e += format_monodix_par(wordmap['new_para'])
    e += '</e>'
    return e


# self test
if __name__ == '__main__':
    fail = False
    for stuff, ape in ApertiumFormatter.stuff2apertium.items():
        if len(ape) < 2:
            continue
        elif ape.startswith('+'):
            if not ape[ape.find('+'):]:
                print("There are conflicting formattings in here!",
                      ape[ape.find('+'):],
                      "is not a valid apertium multichar_symbol!")
                fail = True
        elif ape not in ApertiumFormatter.apertium_multichars:
            print("There are conflicting formattings in here!", ape,
                  "is not a valid apertium multichar_symbol!")
            fail = True
    if fail:
        exit(1)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Functions to format giellatekno style analyses from omorfi data."""

# Author: Omorfi contributors <omorfi-devel@groups.google.com> 2015

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# utils to format apertium style data from omorfi database values

from ..error_logging import fail_formatting_missing_for, just_fail
from .formatter import Formatter
from ..settings import (deriv_boundary, morph_boundary, stub_boundary,
                        weak_boundary, word_boundary)
from ..string_manglers import lexc_escape


class GiellaFormatter(Formatter):
    """Formatter that handles conversions to giellatekno.uit.no conventions."""

    ## Giellatekno style analysis tags
    giella_multichars = {
        '+A',
        '+ABBR',
        '+Abe',
        '+Abl',
        '+Acc',
        '+Act',
        '+ACR',
        '+Ade',
        '+Adp',
        '+Adp+Po',
        '+Adp+Pr',
        '+Adv',
        '+AgPrc',
        '+All',
        '+Card',
        '+CC',
        '+Com',
        '+Comp',
        '+Cmp',
        '+Cond',
        '+ConNeg',
        '+CS',
        '+Dash',
        '+Dem',
        "+Dem",
        '+Der/inint',
        '+Der/nen',
        '+Der/inen',
        '+Der/hko',
        '+Der/ja',
        '+Der/lainen',
        '+Der/llinen',
        '+Der/maisilla',
        '+Der/minen',
        '+Der/nti',
        '+Der/sti',
        "+Der/sti",
        '+Der/tar',
        '+Der/tattaa',
        '+Der/tatuttaa',
        '+Der/tuttaa',
        '+Der/mainen',
        '+Der/isa',
        '+Der/ton',
        '+Der/tse',
        '+Der/ttaa',
        '+Der/ttain',
        '+Der/u',
        '+Der/uus',
        '+Der/vs',
        '+Dial/Finland',
        '+Dial/East',
        '+Dial/Eteläpohjalaiset',
        '+Dial/Häme',
        '+Dial/Keskipohjalaiset',
        '+Dial/North',
        '+Dial/Peräpohjalaiset',
        '+Dial/Savo',
        '+Dial/Southeast',
        '+Dial/Southwest',
        '+Dial/Standard',
        '+Dial/West',
        '+Digit',
        '+Ela',
        '+Err/Orth',
        '+Err/Sub',
        '+Ess',
        '+Foc/han',
        '+Foc/ka',
        '+Foc/kaan',
        '+Foc/kin',
        '+Foc/ko',
        '+Foc/pa',
        '+Foc/s',
        '+Gen',
        '+Ill',
        '+Impv',
        "+Indef",
        '+Ine',
        '+InfA',
        '+InfE',
        '+InfMA',
        '+Inf1',
        '+Inf2',
        '+Inf3',
        '+Ins',
        '+Interj',
        '+Interr',
        "+Interr",
        '+Lat',
        '+Man',
        '+N',
        '+N+Abbr',
        '+Neg',
        '+NegPrc',
        '+Nom',
        '+Num',
        '+OLang/eng',
        '+Opt',
        '+Ord',
        '+Par',
        '+Pass',
        '+Pcle',
        '+Pe4',
        '+Pers',
        "+Pers",
        '+Pl',
        '+Pl1',
        '+Pl2',
        '+Pl3',
        '+Po',
        '+Pos',
        '+Pot',
        '+Pr',
        '+Pref',
        '+PrfPrc',
        '+PrfPrc+Act',
        '+PrfPrc+Pass',
        '+Pron',
        '+Prop',
        '+Prs',
        '+PrsPrc',
        '+PrsPrc+Act',
        '+PrsPrc+Pass',
        '+Prt',
        '+Pst',
        '+Punct',
        '+PxPl1',
        '+PxPl2',
        '+PxPl3',
        '+PxSg1',
        '+PxSg2',
        '+PxSg3',
        '+Px3',
        "+Qnt",
        '+Qst',
        '+Qu',
        '+Quote',
        '+Refl',
        "+Refl",
        '+Rel',
        "+Rel",
        '+Roman',
        '+Sem/Geo',
        '+Sem/Human',
        '+Sem/Org',
        '+Sg',
        '+Sg1',
        '+Sg2',
        '+Sg3',
        '+Suff',
        '+Superl',
        '+Tra',
        '+Trunc',
        '+TruncPrefix',
        'TruncSuffix+',
        '+Use/Arch',
        '+Use/Circ',
        '+Use/Marg',
        '+Use/Rare',
        '+V'
    }

    ## Omorfi to giellatekno tag mapping
    stuff2giella = {"Bc": "#",
                    ".": "",
                    "Aiden": "",
                    "Aien": "",
                    "Aiin": "",
                    "Ain": "",
                    "Ayn": "",
                    "Aän": "",
                    "Aön": "",
                    "Aisiin": "",
                    "Aseen": "",
                    "Aä": "",
                    "Ajä": "",
                    "Atä": "",
                    "Ajen": "",
                    "Aten": "",
                    "Ahin": "",
                    "Ahen": "",
                    "Ahyn": "",
                    "Aihin": "",
                    "Aiä": "",
                    "Ana": "",
                    "Asa": "",
                    "Aitten": "",
                    "Aan": "",
                    "Aen": "",
                    "Ahan": "",
                    "Ahon": "",
                    "Ahun": "",
                    "Aon": "",
                    "Aun": "",
                    "Aa": "",
                    "Aia": "",
                    "Aita": "",
                    "Aja": "",
                    "Ahän": "",
                    "Ahön": "",
                    "Aitä": "",
                    "Ata": "",
                    "ABBREVIATION": "+ABBR",
                    "ABLATIVE": "+Abl",
                    "ACRONYM": "+ACR",
                    "ADESSIVE": "+Ade",
                    "ADJECTIVE": "+A",
                    "ADPOSITION": "+Adp+Po",
                    "ADVERB": "+Adv",
                    "ADVERBIAL": "+CS",
                    "ALLATIVE": "+All",
                    "ARROW": "",
                    "ARTWORK": "",
                    "B-": "+Trunc",
                    "B←": "+TruncPrefix",
                    "B→": "TruncSuffix+",
                    "CARDINAL": "",
                    "Ccmp": "+Comp",
                    "CLAUSE-BOUNDARY": "",
                    "Cma": "+AgPrc",
                    "Cmaisilla": "+Der/maisilla",
                    "Cmaton": "+NegPrc",
                    "Cnut": "+PrfPrc",
                    "COMMA": "",
                    "COMPARATIVE": "+CS",
                    "COMP": "+Comp",
                    "CONJUNCTION": "",
                    "COORDINATING": "+CC",
                    "Cpos": "+Pos",
                    "Csup": "+Superl",
                    "CULTGRP": "+Sem/Human",
                    "Cva": "+PrsPrc",
                    "DASH": "+Dash",
                    "DECIMAL": "",
                    "DEMONSTRATIVE": "+Dem",
                    "DERSTI": "+Der/sti",
                    "DERTTAIN": "+Der/ttain",
                    "DIGIT": "+Digit",
                    "Dinen": "+Der/inint",
                    "Din": "+Superl",
                    "Dja": "+Der/ja",
                    "Dma": "+AgPrc",
                    "Dmaisilla": "+Der/maisilla",
                    "Dmaton": "+NegPrc",
                    "Dminen": "+N",
                    "Dmpi": "+Comp",
                    "Dnut": "+PrfPrc+Act",
                    "Ds": "+Cmp",
                    "Dsti": "+Der/sti",
                    "Dtattaa": "+Der/tattaa",
                    "Dtatuttaa": "+Der/tatuttaa",
                    "Dtava": "+PrsPrc+Pass",
                    "Dttaa": "+Der/ttaa",
                    "Dtar": "+Der/tar",
                    "Dmainen": "+Der/mainen",
                    "Dton": "+Der/ton",
                    "Dllinen": "+Der/llinen",
                    "Dhko": "+Der/hko",
                    "Disa": "+Der/isa",
                    "Dnen": "+Der/nen",
                    "Dtuttaa": "+Der/tuttaa",
                    "Dlainen": "+Der/lainen",
                    "Dttain": "+Der/ttain",
                    "Dtu": "+PrfPrc+Pass",
                    "Du": "+Der/u",
                    "Duus": "+Der/uus",
                    "Dva": "+PrsPrc+Act",
                    "EVENT": "",
                    "FINAL-BRACKET": "",
                    "FINAL-QUOTE": "+Quote",
                    "FIRST": "+Sem/Human",
                    "FTB3MAN": "+Ins",
                    "FTB3man": "+Man",
                    "GEO": "+Sem/Geo",
                    "Ia": "+Inf1",
                    "Ie": "+Inf2",
                    "ILLATIVE": "+Ill",
                    "Ima": "+Inf3",
                    "Iminen": "+N",
                    "INDEFINITE": "+Indef",
                    "INESSIVE": "+Ine",
                    "INITIAL-BRACKET": "",
                    "INITIAL-QUOTE": "+Quote",
                    "INTERJECTION": "+Interj",
                    "INTERROGATIVE": "+Interr",
                    "LAST": "+Sem/Human",
                    "LEMMA-END": "",
                    "LEMMA-START": "",
                    "LOCATIVE": "",
                    "MEDIA": "",
                    "MISC": "",
                    "Ncon": "+ConNeg",
                    "Nneg": "+Neg",
                    "NOUN": "+N",
                    "Npl": "+Pl",
                    "N??": "+Sg",
                    "Nsg": "+Sg",
                    "NUMERAL": "+Num",
                    "NUM": "+Num",
                    "O3": "+Px3",
                    "Opl1": "+PxPl1",
                    "Opl2": "+PxPl2",
                    "ORDINAL": "+Ord",
                    "ORG": "+Sem/Org",
                    "Osg1": "+PxSg1",
                    "Osg2": "+PxSg2",
                    "PARTICLE": "+Pcle",
                    "PE4": "+Pe4",
                    "PERSONAL": "+Pers",
                    "PL1": "+Pl1",
                    "PL2": "+Pl2",
                    "PL3": "+Pl3",
                    "Ppe4": "+Pe4",
                    "Ppl1": "+Pl1",
                    "Ppl2": "+Pl2",
                    "Ppl3": "+Pl3",
                    "PREPOSITION": "+Adp+Pr",
                    "PRODUCT": "",
                    "PRONOUN": "+Pron",
                    "PROPER": "+Prop",
                    "PROPN": "+Prop",
                    "Psg1": "+Sg1",
                    "Psg2": "+Sg2",
                    "Psg3": "+Sg3",
                    "PUNCTUATION": "+Punct",
                    "Qhan": "+Foc/han",
                    "Qkaan": "+Foc/kaan",
                    "Qka": "+Foc/ka",
                    "Qkin": "+Foc/kin",
                    "Qko": "+Foc/ko",
                    "Qpa": "+Foc/pa",
                    "Qs": "+Foc/s",
                    "QUALIFIER": "+A",
                    "QUANTOR": "+Qnt",
                    "RECIPROCAL": "",
                    "REFLEXIVE": "+Refl",
                    "RELATIVE": "+Rel",
                    "ROMAN": "+Roman",
                    ".sent": "",
                    "SENTENCE-BOUNDARY": "",
                    "SG1": "+Sg1",
                    "SG2": "+Sg2",
                    "SG3": "+Sg3",
                    "SPACE": "",
                    "SUPERL": "+Superl",
                    "SUFFIX": "",
                    "Tcond": "+Cond",
                    "Timp": "+Impv",
                    "Topt": "+Opt",
                    "Tpast": "+Pst",
                    "Tpot": "+Pot",
                    "Tpres": "+Prs",
                    "Uarch": "+Use/Arch",
                    "Udial": "+Dial/Finland",
                    "Unonstd": "+Err/Orth",
                    "UNSPECIFIED": "+Pcle",
                    "Urare": "+Use/Marg",
                    "Vact": "+Act",
                    "VERB": "+V",
                    "Vpss": "+Pass",
                    "Xabe": "+Abe",
                    "Xabl": "+Abl",
                    "Xacc": "+Acc",
                    "Xade": "+Ade",
                    "Xall": "+All",
                    "Xcom": "+Com",
                    "Xela": "+Ela",
                    "Xess": "+Ess",
                    "Xgen": "+Gen",
                    "Xill": "+Ill",
                    "Xine": "+Ine",
                    "Xins": "+Ins",
                    "Xlat": "+Lat",
                    "X???": "+Nom",
                    "Xnom": "+Nom",
                    "Xpar": "+Par",
                    "Xtra": "+Tra",
                    "XForeign": "+OLang/eng",
                    "X": "",
                    "": ""
                    }

    def __init__(self, verbosity=False):
        """Create a giellatekno tag formatter with given verbosity.

        @param verbosity set to true to disable output printng
        """
        ## Verbosity
        self.verbosity = verbosity
        fail = False
        for _, giella in self.stuff2giella.items():
            if len(giella) < 2:
                continue
            elif giella not in self.giella_multichars:
                just_fail("There are conflicting formattings in here!\n" +
                          giella +
                          " is not a valid defined giella multichar_symbol!")
                fail = True
        if fail:
            ## Whether tags match reality
            self.tainted = True

    def stuff2lexc(self, stuff):
        """Convert omorfi analysis to giellatekno's lexc.

        @return lexc-formatted string containing giellatekno tag(s)
        """
        if stuff == '0':
            return "0"
        elif stuff in self.stuff2giella:
            return self.stuff2giella[stuff]
        else:
            fail_formatting_missing_for(stuff, "giella.1")
            return ""

    def analyses2lexc(self, anals, surf):
        """Convert omorfi analyses and lexeme into giellatekno format.

        @return lexc-formatted string containing giellatekno tags and lemma
        """
        giellastring = ""
        for anal in anals.split('|'):
            if anal == '@@COPY-STEM@@':
                giellastring += lexc_escape(surf)
            elif anal.startswith('@@LITERAL:') and anal.endswith('@@'):
                giellastring += lexc_escape(anal[len('@@LITERAL:'):-len('@@')])
            else:
                giellastring += self.stuff2lexc(anal)
        return giellastring

    def continuation2lexc(self, anals, surf, cont):
        """Create lexical entry for omorfi continuation class data

        @return lexc-date for omorfi continuation class
        """
        giellastring = self.analyses2lexc(anals, surf)
        if 'DIGITS_' in cont and not ('BACK' in cont or 'FRONT' in cont):
            giellastring = lexc_escape(surf) + giellastring
        surf = lexc_escape(surf.replace(morph_boundary, ">")
                           .replace(deriv_boundary, "»")
                           .replace(word_boundary, "")
                           .replace(stub_boundary, ""))
        return "%s:%s\t%s ;\n" % (giellastring, surf, cont)

    def wordmap2lexc(self, wordmap):
        '''
        format string for canonical giella format for morphological analysis

        @return lexc-formatted lexical entry for a giellatekno word.
        '''
        wordmap['analysis'] = lexc_escape(
            wordmap['lemma'].replace(word_boundary, '#'))
        if wordmap['pos'] == 'CONJUNCTIONVERB':
            if wordmap['lemma'] == 'eikä':
                wordmap['lemma'] = 'ei'
                wordmap['analysis'] = self.stuff2lexc('COORDINATING') + \
                    self.stuff2lexc('Nneg')
            else:
                wordmap['analysis'] = self.stuff2lexc('ADVERBIAL') + \
                    self.stuff2lexc('Nneg')
        else:
            wordmap['analysis'] += self.stuff2lexc(wordmap['pos'])
        if wordmap['is_proper']:
            wordmap['analysis'] += self.stuff2lexc('PROPER')
            if wordmap['proper_noun_class']:
                wordmap['analysis'] +=\
                    self.stuff2lexc(wordmap['proper_noun_class'])
        if wordmap['particle']:
            for pclass in wordmap['particle'].split('|'):
                wordmap['analysis'] += self.stuff2lexc(pclass)
        if wordmap['symbol']:
            for subcat in wordmap['symbol'].split('|'):
                wordmap['analysis'] += self.stuff2lexc(subcat)
        lex_stub = lexc_escape(wordmap['stub'].replace(word_boundary, "")
                               .replace(weak_boundary, "")
                               .replace(deriv_boundary, "»")
                               .replace(morph_boundary, ">"))
        lexc_line = "%s:%s\t%s\t;" % (wordmap['analysis'], lex_stub,
                                      wordmap['new_para'])
        if 'BLACKLISTED' in wordmap['new_para']:
            return "! ! !" + lexc_line
        else:
            return lexc_line

    def multichars_lexc(self):
        """Create lexc analysis tags declaration in giellatekno format.

        @return lexc-formatted string of giellatekno Mutlichar_symbols block.
        """
        multichars = "Multichar_Symbols\n!! giellatekno multichar set:\n"
        for mcs in self.giella_multichars:
            multichars += mcs + "\n"
        multichars += Formatter.multichars_lexc(self)
        return multichars

    def root_lexicon_lexc(self):
        """Create root lexicon for giellatekno lexc.

        @return lexc-formatted string of giellatekno Root lexicon
        """
        root = Formatter.root_lexicon_lexc(self)
        if True:
            # want co-ordinated hyphens left
            root += "!! LEXICONS that can be co-ordinated hyphen -compounds\n"
            root += self.stuff2lexc('B→') + ':-   NOUN ;\n'
            root += self.stuff2lexc('B→') + ':-   ADJ ;\n'
            root += self.stuff2lexc('B→') + ':-   SUFFIX ;\n'
        return root


# self test
if __name__ == '__main__':
    formatter = GiellaFormatter()
    exit(0)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Functions to format Xerox style regexes from omorfi data."""

# Author: Omorfi contributors <omorfi-devel@groups.google.com> 2015

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# utils to format xerox regexes from omor's lexical data sources.

from ..settings import (deriv_boundary, fin_lowercase, fin_orth_pairs,
                        fin_uppercase, morph_boundary, stub_boundary,
                        weak_boundary, word_boundary)
from .twolc_formatter import twolc_escape


def format_rules_regex(formatter, ruleset):
    """Generate omorfi rlues in Xerox regex format."""
    regexstring = ''
    if ruleset == 'orthographic-variations':
        regexstring += '[ '
        for p in fin_orth_pairs:
            regexstring += twolc_escape(p[0]) + ':' + twolc_escape(p[1]) + \
                ' | ' + twolc_escape(p[0]) + ' | '
        regexstring += '? ]* ;'
    elif ruleset == 'zh':
        regexstring += '[ ž | ž:z 0:h | ž:z::1 ] ;'
    elif ruleset == 'sh':
        regexstring += '[ š | š:s 0:h | š:s::1 ] ;'
    elif ruleset == 'rewrite-tags-ftb3':
        regexstring += '# Remove before compounds:\n'
        regexstring += '[ '
        regexstring += ' -> 0,\n '.join([formatter.stuff2lexc(tag) for tag in
                                         ['ADJ', 'NOUN', 'VERB', 'ACRONYM',
                                          'ABBREVIATION', 'NUM', 'PROPER',
                                          'DIGIT', 'Xnom', 'Xpar', 'Xgen',
                                          'Xine', 'Xela', 'Xill', 'Xade',
                                          'Xabl', 'Xall', 'Xess', 'Xins',
                                          'Xabe', 'Xtra', 'Xcom', 'Nsg',
                                          'Npl']])
        regexstring += '-> 0 || _ ?* %# ]\n'
        regexstring += '.o.\n'
        regexstring += '# Remove V before Prc\n'
        regexstring += '[ ' + formatter.stuff2lexc('VERB') + ' -> 0 || _  [ '
        regexstring += ' | '.join([formatter.stuff2lexc(tag) for tag in
                                   ['Cma', 'Cmaisilla', 'Cnut', 'Cva',
                                    'Cmaton', 'Dma', 'Dnut', 'Dtu', 'Dva',
                                    'Dtava']])
        regexstring += '] ]\n'
        regexstring += '.o.\n'
        regexstring += '# ftb3.1 all pr are optional po\n'
        regexstring += '[ ' + formatter.stuff2lexc('ADP') + ' (->) ' +\
            formatter.stuff2lexc('PREPOSITION') \
            + ']\n'
        regexstring += '.o.\n'
        regexstring += '# stem mangling rules:\n'
        regexstring += '[ %<Del%>→ [ ' + ' | '.join(fin_lowercase) + \
            ' ]* ←%<Del%> -> 0 || _ [ ? - %#]* %# ]\n'
        regexstring += '.o.\n'
        regexstring += '[ ←%<Del%> -> 0, %<Del%>→ -> 0 ]\n'
        regexstring += '.o.\n'
        regexstring += '[ ' + ' | '.join(fin_lowercase) + ']* -> 0 || ' +\
            '[ ' + formatter.stuff2lexc('NOUN') + \
            ' | ' + formatter.stuff2lexc('NUM') + \
            '] [? - %#]* _ [? - %#]* .#. \n'
        regexstring += '.o.\n'
        regexstring += '# Puncts without nom case\n'
        regexstring += '[ ' + formatter.stuff2lexc('Xnom') +\
            ' ' + formatter.stuff2lexc('Nsg') + ' ] -> 0 || ' +\
            formatter.stuff2lexc('PUNCTUATION') + ' _ \n'
        regexstring += '.o.\n'
        regexstring += '# random puncts are abbr\n'
        regexstring += formatter.stuff2lexc('PUNCTUATION') +\
            ' (->) ' + formatter.stuff2lexc('ABBREVIATION') +\
            ' || _ '
        regexstring += '.o.\n'
        regexstring += '# abbrs are nom sg’s too\n'
        regexstring += formatter.stuff2lexc('ABBREVIATION') +\
            ' (->) ' + formatter.stuff2lexc('ABBREVIATION') +\
            ' ' + formatter.stuff2lexc('Xnom') +\
            ' ' + formatter.stuff2lexc('Nsg') +\
            ' || _ '
        regexstring += '.o.\n'
        regexstring += '# suffixes unmarked why of course\n'
        regexstring += '%# %- -> %# || _ \n'
        regexstring += '.o.\n'
        regexstring += '# FTB is a bit silly with dashes\n'
        regexstring += '% Dash -> % EmDash || — ?* _ \n'
        regexstring += '.o.\n'
        regexstring += '% Dash -> % EnDash || – ?* _ \n'
        regexstring += ';\n'
    elif ruleset == 'lemmatise':
        regexstring += '# Remove everything:\n'
        regexstring += '[ '
        regexstring += ' -> 0,\n'.join([formatter.stuff2lexc(tag) for tag in
                                        ['ADJ', 'NOUN', 'VERB', 'ACRONYM',
                                         'ABBREVIATION', 'NUM', 'PROPER',
                                         'DIGIT', 'COORDINATING', 'ADVERBIAL',
                                         'ORDINAL', 'DEMONSTRATIVE',
                                         'PERSONAL', 'INDEFINITE', 'QUANTOR',
                                         'INTERROGATIVE', 'REFLEXIVE',
                                         'RELATIVE', 'PUNCTUATION', 'DASH',
                                         'ROMAN', 'PL1', 'PL2', 'PL3', 'SG1',
                                         'SG2', 'SG3', 'PE4', 'COMP', 'SUPERL',
                                         'UNSPECIFIED', 'PRON', 'INTJ',
                                         'Xnom', 'Xpar', 'Xgen', 'Xine',
                                         'Xela', 'Xill',
                                         'Xade', 'Xabl', 'Xall', 'Xess',
                                         'Xins', 'Xabe',
                                         'Xtra', 'Xcom', 'Nsg', 'Npl',
                                         'Osg1', 'Osg2', 'O3', 'Opl1', 'Opl2',
                                         'Qka', 'Qs', 'Qpa', 'Qko', 'Qkin',
                                         'Qkaan', 'Qhan',
                                         'Vact', 'Vpss',
                                         'Ncon', 'Nneg', 'Dnut', 'Dtu',
                                         'Dva', 'Dtava',
                                         'Ia', 'Ie', 'Ima',
                                         'Tcond', 'Timp', 'Tpast', 'Tpot',
                                         'Tpres',
                                         'Topt'] if formatter.stuff2lexc(tag)])
        regexstring += '-> 0 || _ ] ;\n'
    elif ruleset == 'remove-boundaries':
        regexstring += ' -> 0, '.join([twolc_escape(tag) for tag in
                                       [word_boundary, deriv_boundary,
                                        morph_boundary, stub_boundary,
                                        weak_boundary]]) + '-> 0 || _ ;'
    elif ruleset == 'remove-boundaries-giella':
        regexstring += ' -> 0, '.join([twolc_escape(tag) for tag in
                                       [">", "»"]]) + '-> 0 || _ ;'
    elif ruleset == 'token':
        regexstring += '[ [' + '|'.join(fin_lowercase) + '|' +\
            '|'.join(fin_uppercase) + \
            '| ’ | %\' | %- | %0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 ]*'\
            ' | ? ] ;'
    elif ruleset == 'between-tokens':
        regexstring += '[ %. | %, | %: | %; | %? | %! | %- | %  ] ;'
    elif ruleset == 'token-separator':
        regexstring += '0:"\\n" ;'
    else:
        print("Unknown ruleset", ruleset)
        return None
    return regexstring



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Python data format for one lexeme in omorfi database."""

# Author: Omorfi contributors <omorfi-devel@groups.google.com> 2015

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.


def init_wordmap():
    wordmap = {"upos": None, "lemma": None, "homonym": 0, "new_para": None, "kotus_tn": None,
               "kotus_av": None, "plurale_tantum": None,
               "possessive": None, "clitics": None,
               "is_proper": None, "proper_noun_class": None, "style": None,
               "stub": None, "gradestem": None, "twolstem": None,
               "grade_dir": None, "harmony": None, "is_suffix": None,
               "is_prefix": None, "stem_vowel": None, "stem_diphthong": None,
               "sem": None, "particle": None, "pronunciation": None,
               "boundaries": None, "bracketstub": None, "origin": None,
               "extra_i": False, "extra_e": False, "real_pos": None,
               "symbol": None, "argument": None, "pronoun": None,
               "abbr": None, "lex": None,
               "numtype": None, "prontype": None, "adptype": None,
               "homonym": 0,
               "blacklist": None,
               "pos": None,
               "deletion": None,
               "suffix_regex": None}
    return wordmap


def get_wordmap_fieldnames():
    return ["upos", "lemma", "homonym", "new_para", "kotus_tn", "kotus_av",
            "plurale_tantum", "possessive", "clitics", "is_proper",
            "proper_noun_class", "style", "stub", "gradestem", "twolstem",
            "grade_dir", "harmony", "is_suffix", "is_prefix", "stem_vowel",
            "stem_diphthong", "sem", "particle", "pronunciation",
            "boundaries", "bracketstub", "origin", "extra_i", "extra_e",
            "real_pos", "symbol", "argument", "pronoun", "abbr",
            "lex", "numtype", "prontype", "adptype", "blacklist", "pos",
            "deletion", "suffix_regex"]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# (c) Omorfi contributors <omorfi-devel@groups.google.com> 2015
# see AUTHORS file in top-level dir of this project, or
# <https://github.com/flammie/omorfi/wiki/AUTHORS>

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""
An interface for formatting, deformatting etc. tags and stuff.
"""

from abc import ABCMeta, abstractmethod

from ..settings import common_multichars, version_id_easter_egg


class Formatter(metaclass=ABCMeta):

    """
    An abstract base class for omorfi objects doing format conversions and
    such string mangling. The implememting classes are mainly tag-set
    variants, for conversions between different syntaxes there are helper
    functions. The better the conversions between stuffs are, the better the
    raw and python-based analysers are.

    The formats to convert include:

    * from omorfi's stuff strings
    * to lexc
    * to monodix
    * from wordmap
    * from continuation record (a tsv row)
    """

    def __init__(self, verbosity=False, **kwargs):
        """Construct formatter with given verbosity.
        
        @param verbosity whether to print messages while processing
        @param kwargs keyword map to determine what automata to load
        """
        self._verbosity = verbosity

    def copyright_lexc(self):
        """Return copyright declaration in lexc format.
        
        @return string holding current copyright declaration in lexc comment.
        """
        return "! Copyright 2015 Omorfi Contributors, GNU GPLv3"

    @abstractmethod
    def multichars_lexc(self):
        """Return multichar declaration in lexc format"""
        multichars = "!! Following specials exist in all versions of omorfi\n"
        for mcs in sorted(common_multichars):
            multichars += mcs + "\n"
        return multichars

    @abstractmethod
    def root_lexicon_lexc(self):
        """Return root lexicon in lexc format"""
        root = "LEXICON Root\n"
        root += """!! LEXICONS per class
    0   NOUN ;
    0   ADJ ;
    0   VERB    ;
    0   NUM ;
    0   DIGITS ;
    0   PRON    ;
    0   ADP    ;
    0   ADV    ;
    0   INTJ ;
    0   PUNCT ;
    0   SYM ;
    0   CCONJ ;
    0   SCONJ ;
    0   CCONJ|VERB ;
    0   PROPN ;
    0   ACRONYM ;
    0   ABBREVIATION    ;
    0   AUX    ;
    0   DET    ;
    0   X    ;
    """
        root += version_id_easter_egg + ':__omorfi # ;\n'
        return root

    @abstractmethod
    def wordmap2lexc(self, wordmap):
        """Turn wordmap into lexc string valid for insides of LEXICON"""
        pass

    @abstractmethod
    def continuation2lexc(self, fields):
        """Turn continuation into lexc string valid for insides of LEXICON."""
        pass


def main():
    """Self test exmaple"""
    exit(0)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from argparse import ArgumentParser, FileType
# CLI stuff
from sys import stderr, stdin, stdout
# statistics
from time import perf_counter, process_time

# omorfi
from omorfi import Omorfi, Token


def try_analyses_ftb(original, wordn, token, outfile, hacks=None):
    best = None
    highest = -1
    for i, anal in enumerate(token.analyses):
        pos = anal.format_xpos_ftb()
        feats = anal.format_feats_ftb()
        lemmas = "#".join(anal.get_lemmas())
        score = 0
        if pos == original[3]:
            score += 10
        if feats == original[5]:
            score += 10
        if lemmas == original[2]:
            score += 10
        if score > highest:
            best = i
    print(token.printable_ftb3(best), file=outfile)


def print_analyses_ftb(wordn, token, outfile, hacks=None):
    print(token.printable_ftb3(), file=outfile)


def main():
    """Invoke a simple CLI analyser."""
    a = ArgumentParser()
    a.add_argument('-a', '--analyser', metavar='AFILE',
                   help="read analyser model from AFILE",
                   required=True)
    a.add_argument('-i', '--input', metavar="INFILE", type=open,
                   dest="infile", help="source of analysis data")
    a.add_argument('-v', '--verbose', action='store_true',
                   help="print verbosely while processing")
    a.add_argument('-o', '--output', metavar="OUTFILE", dest="outfile",
                   help="print output into OUTFILE", type=FileType('w'))
    a.add_argument('-x', '--statistics', metavar="STATFILE", dest="statfile",
                   help="print statistics to STATFILE", type=FileType('w'))
    a.add_argument('-O', '--oracle', action='store_true',
                   help="match to values in input when parsing if possible")
    a.add_argument('-X', '--frequencies', metavar="FREQDIR",
                   help="read frequencies from FREQDIR/*.freqs")
    a.add_argument('--debug', action='store_true',
                   help="print lots of debug info while processing")
    options = a.parse_args()
    if options.verbose:
        print("Printing verbosely")
    omorfi = Omorfi(options.verbose)
    if options.analyser:
        if options.verbose:
            print("reading analyser model", options.analyser)
        omorfi.load_analyser(options.analyser)
    else:
        print("analyser is needed to ftb3", file=stderr)
        exit(4)
    if not options.infile:
        print("reading from <stdin>")
        options.infile = stdin
    if options.verbose:
        print("analysing", options.infile.name)
    if not options.outfile:
        options.outfile = stdout
    if options.verbose:
        print("writing to", options.outfile.name)
    if not options.statfile:
        options.statfile = stdout

    if options.frequencies:
        with open(options.frequencies + '/lexemes.freqs') as lexfile:
            omorfi.load_lexical_frequencies(lexfile)
        with open(options.frequencies + '/omors.freqs') as omorfile:
            omorfi.load_omortag_frequencies(omorfile)

    # statistics
    realstart = perf_counter()
    cpustart = process_time()
    tokens = 0
    unknowns = 0
    sentences = 0
    for line in options.infile:
        fields = line.strip().split('\t')
        if len(fields) == 10:
            # ftb is 10 field format
            tokens += 1
            try:
                index = int(fields[0])
            except ValueError:
                print("Cannot figure out token index", fields[0], file=stderr)
                exit(1)
            token = Token(fields[1])
            token.pos = int(fields[0])
            omorfi.analyse(token)
            if token.is_oov():
                unknowns += 1
                omorfi.guess(token)
            if options.oracle:
                try_analyses_ftb(fields, index, token, options.outfile)
            else:
                print_analyses_ftb(index, token, options.outfile)
        elif line.startswith('<') and line.rstrip().endswith('>'):
            print(line.strip(), file=options.outfile)
        elif not line or line.strip() == '':
            # retain exactly 1 empty line between sents
            print(file=options.outfile)
            sentences += 1
        else:
            print("Error in ftb3 format: '", line, "'", file=stderr)
            exit(1)
    cpuend = process_time()
    realend = perf_counter()
    print("Tokens:", tokens, "Sentences:", sentences,
          file=options.statfile)
    print("Unknowns / OOV:", unknowns, "=",
          unknowns / tokens * 100 if tokens != 0 else 0,
          "%", file=options.statfile)
    print("CPU time:", cpuend - cpustart, "Real time:", realend - realstart,
          file=options.statfile)
    print("Tokens per timeunit:", tokens / (realend - realstart),
          file=options.statfile)
    exit(0)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
A lightweight container for whole texts. Contains tokens.
"""

from .token import Token


class Doc:
    """Doc contains tokens consisting a text or corpus.

    Doc is typically e.g. one file tokenised and/or parsed. Contains tokens,
    which make up sentences and stuff.
    """

    def __init__(self):
        """Create an empty document"""
        ## underlying raw omor analysis
        self.tokens = []
        ## sentences
        self.sents = []

    def __getitem__(self, index):
        """Just pick token from the list."""
        return self.tokens[index]

    def __str__(self):
        s = 'Doc: '
        if len(self.tokens) > 10:
            s += self.tokens[:10] + ', ...'
        else:
            s += self.tokens
        return s

    def get_sentence(self, index):
        '''Get a sentence from doc.

        If doc contains nontokens separating sentences...
        '''
        offsets = self.sents[index]
        return self.tokens[offsets[0]:offsets[1]]

    def add(self, tokens):
        '''Add tokens to the documentation.

        Adds sentences if they are separated by proper nontokens.
        '''
        sent_start = False
        sent_end = -1
        for token in tokens:
            self.tokens.append(token)
            if not token.nontoken:
                if not sent_start or sent_start < sent_end:
                    sent_start = len(self.tokens)
            else:
                if sent_start:
                    sent_end = len(self.tokens)
                    self.sents.append([sent_start, sent_end])

    def write(self, f):
        '''Writes self in some format into a file-like object.

        Experimental.
        '''
        print('"omorfi.Doc": { "tokens": [', file=f)
        first = True
        for token in self.tokens:
            if not first:
                print(",", file=f)
            print(token, end='', file=f)
            first = False
        print('\n], "sents": [', file=f)
        first = True
        for sent in self.sents:
            if not first:
                print(",", file=f)
            print('[', sent[0], ', ', sent[1], ']', end='', file=f)
            first = False
        print('\n]}', file=f)

    @staticmethod
    def read(f):
        '''Reads self from a file-like object.'''
        line = next(f)
        if line.strip() != '"omorfi.Doc": { "tokens": [':
            print("Error blerb:", line.strip())
            exit(1)
        doc = Doc()
        in_sents = False
        for line in f:
            if line.strip().endswith("]}"):
                continue
            elif in_sents:
                first, last = line.strip().rstrip(",").rstrip("]"). \
                    strip("[").strip().split(", ")
                doc.sents.append([first, last])
            elif '"sents":' in line:
                in_sents = True
            elif '"omorfi.Token":' in line:
                token = Token.fromstr(line.strip().rstrip(","))
                doc.tokens.append(token)
        return doc



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Functions to format apertium style analyses from omorfi data."""

# Author: Omorfi contributors <omorfi-devel@groups.google.com> 2015

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# utils to format apertium style data from omorfi database values

from ..error_logging import fail_formatting_missing_for
from .formatter import Formatter
from .no_tags_formatter import NoTagsFormatter
from ..settings import optional_hyphen, word_boundary
from ..string_manglers import lexc_escape


class LabeledSegmentsFormatter(Formatter):
    """Handles conversions for morph segmenter with morph labels.
    Experimental.
    """

    ## labels for segments
    multichars = {
        '[ADJ]',
        '[VERB]',
        '[NOUN]',
        '[PART]',
        '[PRON]',
        '[NUM]',
        '[PROPN]',
        '[INTJ]',
        '[CCONJ]',
        '[SCONJ]',
        '[ADV]',
        '[ADP]',
        '[POST]',
        '[PREP]',
        '[PUNCT]',
        '[NOM]',
        '[PAR]',
        '[GEN]',
        '[INE]',
        '[ELA]',
        '[ILL]',
        '[ADE]',
        '[ABL]',
        '[ALL]',
        '[ESS]',
        '[INS]',
        '[ABE]',
        '[TRA]',
        '[COM]',
        '[LAT]',
        '[ACC]',
        '[SG]',
        '[PL]',
        '[POSSG1]',
        '[POSSG2]',
        '[POSPL1]',
        '[POSPL2]',
        '[POSSP3]',
        '[TRUNC]',
        '[PRES]',
        '[PAST]',
        '[COND]',
        '[POTN]',
        '[IMPV]',
        '[SG1]',
        '[SG2]',
        '[SG3]',
        '[PL1]',
        '[PL2]',
        '[PL3]',
        '[PE4]',
        '[CONNEG]',
        '[NEG]',
        '[ACTV]',
        '[PASV]',
        '[INFA]',
        '[INFE]',
        '[INFMA]',
        '[INFMAISILLA]',
        '[PCPNUT]',
        '[PCPVA]',
        '[PCPMA]',
        '[PCPMATON]',
        '[POS]',
        '[COMP]',
        '[SUPER]',
        '[hAn]',
        '[kAAn]',
        '[kin]',
        '[kO]',
        '[pA]',
        '[s]',
        '[kA]',
        '[FORGN]'}

    ## convert omorfi analyses to segment labels
    stuff2labels = {"Bc": "{wB}",
                    ".sent": "",
                    ".": ".",
                    "Aa": "",
                    "Aan": "",
                    "ABBREVIATION": "",
                    "ABESSIVE": "[ABE]",
                    "ABLATIVE": "[ABL]",
                    "ACRONYM": "",
                    "ADESSIVE": "[ADE]",
                    "ADJ": "[ADJ]",
                    "ADJECTIVE": "[ADJ]",
                    "ADP": "[ADP]",
                    "ADV": "[ADV]",
                    "ADVERBIAL": "",
                    "Aen": "",
                    "Ahan": "",
                    "Ahen": "",
                    "Ahin": "",
                    "Ahon": "",
                    "Ahun": "",
                    "Ahyn": "",
                    "Ahän": "",
                    "Ahön": "",
                    "Aia": "",
                    "Aiden": "",
                    "Aien": "",
                    "Aihin": "",
                    "Aiin": "",
                    "Ain": "",
                    "Aisiin": "",
                    "Aita": "",
                    "Aitten": "",
                    "Aitä": "",
                    "Aiä": "",
                    "Aja": "",
                    "Ajen": "",
                    "Ajä": "",
                    "ALLATIVE": "[ALL]",
                    "Ana": "",
                    "Aon": "",
                    "ARROW": "",
                    "Asa": "",
                    "Aseen": "",
                    "Ata": "",
                    "Aten": "",
                    "Atä": "",
                    "Aun": "",
                    "Ayn": "",
                    "Aä": "",
                    "Aän": "",
                    "Aön": "",
                    "B-": "[TRUNC]",
                    "B←": "[TRUNC]",
                    "B→": "[TRUNC]",
                    "CARDINAL": "",
                    "ORDINAL": "[ORD]",
                    "Ccmp": "[COMP]",
                    "CLAUSE-BOUNDARY": "",
                    "Cma": "[PCPMA]",
                    "Cmaisilla": "",
                    "Cmaton": "[PCPMATON]",
                    "Cnut": "[PCPNUT]",
                    "COMMA": "",
                    "COMPARATIVE": "",
                    "COMP": "[COMP]",
                    "CCONJ": "[CCONJ]",
                    "COORDINATING": "",
                    "Cpos": "[POS]",
                    "Csup": "[SUPER]",
                    "Cva": "[PCPVA]",
                    "DASH": "",
                    "DECIMAL": "",
                    "DEMONSTRATIVE": "",
                    "DERSTI": "",
                    "DERTTAIN": "",
                    "DIGIT": "",
                    "Din": "",
                    "Dinen": "",
                    "Dja": "",
                    "Dma": "[PCPMA]",
                    "Dmaisilla": "[INFMAISILLA]",
                    "Dmaton": "[PCPMATON]",
                    "Dminen": "",
                    "Dmpi": "",
                    "Dnut": "[PCPNUT]",
                    "Ds": "",
                    "Dsti": "",
                    "Dtattaa": "",
                    "Dtatuttaa": "",
                    "Dtava": "[PCPVA]",
                    "Dttaa": "",
                    "Dttain": "",
                    "Dtu": "[PCPNUT]",
                    "Du": "",
                    "Duus": "",
                    "Dva": "[PCPVA]",
                    "ELATIVE": "[ELA]",
                    "FINAL-BRACKET": "",
                    "FINAL-QUOTE": "",
                    "FTB3man": "",
                    "FTB3MAN": "",
                    "GENITIVE": "[GEN]",
                    "Ia": "[INFA]",
                    "Ie": "[INFE]",
                    "ILLATIVE": "[ILL]",
                    "Ima": "[INFMA]",
                    "Iminen": "",
                    "INDEFINITE": "",
                    "INESSIVE": "[INE]",
                    "INITIAL-BRACKET": "",
                    "INITIAL-QUOTE": "",
                    "INSTRUCTIVE": "",
                    "INTERROGATIVE": "",
                    "INTJ": "[INTJ]",
                    "LATIVE": "",
                    "LEMMA-END": "",
                    "LEMMA-START": "",
                    "LOCATIVE": "",
                    "Ncon": "[CONNEG]",
                    "Nneg": "",
                    "NOUN": "[NOUN]",
                    "Npl": "[PL]",
                    "N??": "",
                    "Nsg": "[SG]",
                    "NUMERAL": "[NUM]",
                    "NUM": "[NUM]",
                    "O3": "[POSSP3]",
                    "Opl1": "[POSPL1]",
                    "Opl2": "[POSPL2]",
                    "Osg1": "[POSSG1]",
                    "Osg2": "[POSSG2]",
                    "PARTICLE": "",
                    "PARTITIVE": "[PAR]",
                    "PE4": "[PE4]",
                    "PERSONAL": "[PERS]",
                    "PL1": "[PL1]",
                    "PL2": "[PL2]",
                    "PL3": "[PL3]",
                    "Ppe4": "[PE4]",
                    "Ppl1": "[PL1]",
                    "Ppl2": "[PL2]",
                    "Ppl3": "[PL3]",
                    "PREPOSITION": "[PREP]",
                    "PRONOUN": "[PRON]",
                    "PRON": "[PRON]",
                    "PROPER": "[PROPN]",
                    "Psg1": "[SG1]",
                    "Psg2": "[SG2]",
                    "Psg3": "[SG3]",
                    "PUNCTUATION": "[PUNCT]",
                    "Qhan": "[HAN]",
                    "Qkaan": "[KAAN]",
                    "Qka": "[KA]",
                    "Qkin": "[KIN]",
                    "Qko": "[KO]",
                    "Qpa": "[PA]",
                    "Qs": "[S]",
                    "QUALIFIER": "",
                    "QUANTIFIER": "",
                    "QUANTOR": "",
                    "RECIPROCAL": "",
                    "REFLEXIVE": "",
                    "RELATIVE": "",
                    "ROMAN": "",
                    "SCONJ": "[SCONJ]",
                    "SENTENCE-BOUNDARY": "",
                    "SEPARATIVE": "",
                    "SG1": "[SG1]",
                    "SG2": "[SG2]",
                    "SG3": "[SG3]",
                    "SPACE": "",
                    "SUFFIX": "",
                    "SUPERL": "[SUPER]",
                    "Tcond": "[COND]",
                    "Timp": "[IMPV]",
                    "Topt": "",
                    "Tpast": "[PAST]",
                    "Tpot": "[POTN]",
                    "Tpres": "[PRES]",
                    "Uarch": "",
                    "Udial": "",
                    "Unonstd": "",
                    "UNSPECIFIED": "",
                    "Urare": "",
                    "Vact": "[ACTV]",
                    "VERB": "[VERB]",
                    "Vpss": "[PASV]",
                    "X": "",
                    "Xabe": "[ABE]",
                    "Xabl": "[ABL]",
                    "Xacc": "[ACC]",
                    "Xade": "[ADE]",
                    "Xall": "[ALL]",
                    "Xcom": "[COM",
                    "Xela": "[ELA]",
                    "Xess": "[ESS]",
                    "XForeign": "",
                    "Xgen": "[GEN]",
                    "Xill": "[ILL]",
                    "Xine": "[INE]",
                    "Xins": "[INS]",
                    "Xlat": "",
                    "X???": "",
                    "Xnom": "[NOM]",
                    "Xpar": "[PAR]",
                    "Xtra": "[TRA]",
                    "": ""
                    }

    def __init__(self, verbose=True, **kwargs):
        """Create a segment label formatter with given verbosity and options.

        @param verbose set to false to disable output logging
        """
        ## verbosity
        self.verbose = verbose

    def stuff2lexc(self, stuff, _):
        """Convert omorfi analysis into segment labels

        @return lexc-formatted string of labeled segments
        """
        if stuff == '@@COPY-STEM@@':
            # we already copy stem
            return ''
        elif stuff.startswith('@@LITERAL:') and stuff.endswith('@@'):
            # we already copy stem
            return ''
        elif stuff in self.stuff2labels:
            return self.stuff2labels[stuff]
        else:
            if self.verbose:
                fail_formatting_missing_for(stuff, "labeled segements")
            return "ERRORMACRO"

    def analyses2lexc(self, anals, stem):
        """Convert omorfi analysis string and stem into labeled segments.

        @return lexc-formatted string of labeled segments
        """
        apestring = ''
        for i in anals.split('|'):
            apestring += self.stuff2lexc(i, stem)
        return apestring

    def continuation2lexc(self, anals, surf, cont):
        """Create lexc continuation class for omorfi morphology informations.

        @return lexc-formatted entry for continuation class data
        """
        # interleave tags and segments
        analstring = lexc_escape(surf)
        analstring += self.analyses2lexc(anals, surf)
        return "%s:%s\t%s ;\n" % (analstring, lexc_escape(surf), cont)

    def wordmap2lexc(self, wordmap):
        """Format omorfi lexical data into labeled stem segments in lexc.

        @return lexc-formatted lexical entry
        """
        if wordmap['lemma'] == ' ':
            return ''
        wordmap['analysis'] = lexc_escape(wordmap['stub'])
        wordmap['analysis'] += "{STUB}" + "[" + wordmap['upos'] + "]"
        retvals = ""
        wordmap['stub'] = wordmap['stub'].replace(
            word_boundary, optional_hyphen)
        wordmap['stub'] = lexc_escape(wordmap['stub'])
        if 'BLACKLIST' in wordmap['new_para']:
            return "! ! %s:%s\t%s\t;\n" % (wordmap['analysis'],
                                           wordmap['stub'],
                                           wordmap['new_para'])
        else:
            return "%s:%s\t%s\t;\n" % (wordmap['analysis'], wordmap['stub'],
                                       wordmap['new_para'])

        return retvals

    def multichars_lexc(self):
        """Format labels as lexc block

        @return lexc-format multichar symbols definition for segment labels
        """
        multichars = "Multichar_Symbols\n"
        for mc in self.multichars:
            multichars += lexc_escape(mc) + "\n"
        multichars += Formatter.multichars_lexc(self)
        return multichars

    def root_lexicon_lexc(self):
        """Create root lexicon for labeled segments

        @return lexc-format Root lexicon for labeled morph segments
        """
        root = Formatter.root_lexicon_lexc(self)
        return root


# self test
if __name__ == '__main__':
    formatter = NoTagsFormatter()
    exit(0)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Functions to guess omorfi features from partial lexical data."""

# Author: Omorfi contributors <omorfi-devel@groups.google.com> 2015

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ..error_logging import fail_guess_because


def guess_pos_from_newpara(wordmap):
    if wordmap['pos']:
        return wordmap
    new_para = wordmap['new_para']
    wordmap['pos'] = new_para[:new_para.find('_')]


def guess_grade_dir_from_ktn(wordmap):
    '''Record gradation direction based on kotus class or stem syllable.'''
    if not wordmap['kotus_tn']:
        return wordmap
    if wordmap['grade_dir']:
        return wordmap
    tn = int(wordmap['kotus_tn'])
    if tn in range(1, 32) or tn in range(52, 62) or tn in range(76, 79):
        wordmap['grade_dir'] = 'weaken'
    elif tn in range(32, 50) or tn in range(62, 76):
        wordmap['grade_dir'] = 'strengthen'
    elif tn in [0, 99, 101, 999]:
        wordmap['grade_dir'] = False
    elif tn in [1007, 1008, 1009, 1010, 1024, 1026, 1067, 1099, 1101]:
        wordmap['grade_dir'] = 'weaken'
    else:
        fail_guess_because(wordmap, [], ["0-78", 99, 101, 1007, 1008, 1009,
                                         1010, 1024, 1026, 1067, 1099, 1101])
    return wordmap


def guess_stem_features_ktn(wordmap):
    '''Record guessable features based on kotus classification, such as stem
    variants, harmony etc. This is especially for those features kotus
    classification has collapsed that are hard for us.
    '''
    if not wordmap['kotus_tn']:
        return wordmap
    tn = int(wordmap['kotus_tn'])
    if tn in [5, 6]:
        if wordmap['lemma'].endswith('i'):
            wordmap['extra_i'] = True
    elif tn in range(9, 16) or tn in [1009, 1010]:
        if wordmap['lemma'].endswith('a'):
            wordmap['stem_vowel'] = 'a'
        elif wordmap['lemma'].endswith('ä'):
            wordmap['stem_vowel'] = 'ä'
    elif tn in range(17, 19) or tn == 20:
        if wordmap['lemma'].endswith('a'):
            wordmap['stem_vowel'] = 'a'
        elif wordmap['lemma'].endswith('e'):
            wordmap['stem_vowel'] = 'e'
        elif wordmap['lemma'].endswith('i'):
            wordmap['stem_vowel'] = 'i'
        elif wordmap['lemma'].endswith('o'):
            wordmap['stem_vowel'] = 'o'
        elif wordmap['lemma'].endswith('u'):
            wordmap['stem_vowel'] = 'u'
        elif wordmap['lemma'].endswith('y'):
            wordmap['stem_vowel'] = 'y'
        elif wordmap['lemma'].endswith('ä'):
            wordmap['stem_vowel'] = 'ä'
        elif wordmap['lemma'].endswith('ö'):
            wordmap['stem_vowel'] = 'ö'
    elif tn == 19:
        if wordmap['lemma'].endswith('uo'):
            wordmap['stem_diphthong'] = 'uo'
        elif wordmap['lemma'].endswith('yö'):
            wordmap['stem_diphthong'] = 'yö'
        elif wordmap['lemma'].endswith('ie'):
            wordmap['stem_diphthong'] = 'ie'
    elif tn == 47:
        if wordmap['lemma'].endswith('ut'):
            wordmap['stem_vowel'] = 'u'
        elif wordmap['lemma'].endswith('yt'):
            wordmap['stem_vowel'] = 'y'
    elif tn == 64:
        if wordmap['lemma'].endswith('uoda'):
            wordmap['stem_diphthong'] = 'uo'
        elif wordmap['lemma'].endswith('yödä'):
            wordmap['stem_diphthong'] = 'yö'
        elif wordmap['lemma'].endswith('iedä'):
            wordmap['stem_diphthong'] = 'ie'
    return wordmap


def guess_harmony(wordmap):
    '''Guess word's harmony based on lemma, using trivial last harmony vowel
    or front algorithm.
    '''
    if wordmap['harmony']:
        return wordmap
    if not wordmap['kotus_tn']:
        tn = 0
    else:
        tn = int(wordmap['kotus_tn'])
    if tn in range(52, 79) or wordmap['pos'] == 'VERB':
        if wordmap['lemma'].endswith('ä'):
            wordmap['harmony'] = 'front'
        elif wordmap['lemma'].endswith('a'):
            wordmap['harmony'] = 'back'
        elif wordmap['lemma'].endswith('ei') and tn == 1099:
            wordmap['harmony'] = 'front'
        else:
            fail_guess_because(wordmap, ["VERB"], ["a", "ä", "ei"])
            exit(1)
    elif wordmap['pronunciation']:
        lastbound = -1
        for bound in ['{WB}', '{XB}', '_', '#', ' ', '-']:
            b = wordmap['pronunciation'].rfind(bound)
            if b > lastbound:
                lastbound = b
        lastback = lastbound
        for back in ['a', 'o', 'u', 'A', 'O', 'U', 'á', 'à', 'ą', 'ô', 'û']:
            b = wordmap['pronunciation'].rfind(back)
            if b > lastback:
                lastback = b
        lastfront = lastbound
        for front in ['ä', 'ö', 'y', 'Ä', 'Ö', 'Y', 'ü']:
            f = wordmap['pronunciation'].rfind(front)
            if f > lastfront:
                lastfront = f
        if lastfront == lastbound and lastback == lastbound:
            wordmap['harmony'] = 'front'
        elif lastfront > lastback:
            wordmap['harmony'] = 'front'
        elif lastback > lastfront:
            wordmap['harmony'] = 'back'
    else:
        fail_guess_because(wordmap, [""], ["pronunciation"])
        exit(1)
    return wordmap


def guess_pronunciation(wordmap):
    if wordmap['pronunciation']:
        return wordmap
    wordmap['pronunciation'] = wordmap['stub']
    if wordmap['stem_vowel']:
        wordmap['pronunciation'] += wordmap['stem_vowel']
    elif wordmap['stem_diphthong']:
        wordmap['pronunciation'] += wordmap['stem_diphthong']
    return wordmap


def guess_gradestem(wordmap):
    if wordmap['stem_vowel']:
        wordmap['gradestem'] += '{^' + wordmap['stem_vowel'] + '}'
    if wordmap['harmony']:
        wordmap['gradestem'] += '{^' + wordmap['harmony'] + '}'
    if wordmap['kotus_av']:
        wordmap['gradestem'] += '{' + wordmap['kotus_av'] + '}'
    if wordmap['plurale_tantum'] == 'obligatory':
        wordmap['gradestem'] += '@U.PLURALE.TANTUM@'
    elif wordmap['plurale_tantum'] == 'common':
        wordmap['gradestem'] += '{PLT?}'
    return wordmap


def guess_bound_morphs(wordmap):
    if wordmap['stub'] in ['-', '--']:
        return wordmap
    if wordmap['stub'].endswith('-'):
        wordmap['is_prefix'] = True
        wordmap['stub'] = wordmap['stub'][:-1]
    elif wordmap['stub'].startswith('-'):
        wordmap['is_suffix'] = True
        wordmap['stub'] = wordmap['stub'][1:]
    return wordmap



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Functions to format apertium style analyses from omorfi data."""

# Author: Omorfi contributors <omorfi-devel@groups.google.com> 2015

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# utils to format apertium style data from omorfi database values

from ..error_logging import fail_formatting_missing_for, just_fail
from .formatter import Formatter
from ..settings import weak_boundary, word_boundary
from ..string_manglers import lexc_escape


class ApertiumFormatter(Formatter):
    """Formatter that handles conversions for apertium's conventions."""

    ## All apertium multichar symbols
    apertium_multichars = {
        "-",
        "",
        "+",
        "adj",
        "abbr",
        "abe",
        "abl",
        "acc",
        "acr",
        "actv",
        "ade",
        "adv",
        "agent",
        "al",
        "all",
        "ant",
        "card",
        "cnjcoo",
        "cnjcoo><vblex",
        "cnjsub",
        "cnjadv",
        "cog",
        "com",
        "comp",
        "cni",
        "conneg",
        "cmp",
        "cmp-split",
        "compound-only-L",
        "compound-R",
        "def",
        "det",
        "dem",
        "ela",
        "enc",
        "ess",
        "f",
        "gen",
        "ij",
        "ill",
        "imp",
        "impers",
        "ind",
        "ine",
        "infa",
        "infe",
        "infma",
        "infminen",
        "ins",
        "itg",
        "lat",
        "loc",
        "m",
        "mf",
        "n",
        "neg",
        "nom",
        "np",
        "num",
        "ord",
        "p1",
        "p2",
        "p3",
        "p1><pl",
        "p1><sg",
        "p2><pl",
        "p2><sg",
        "p3><pl",
        "p3><sg",
        "par",
        "part",
        "past",
        "pasv",
        "pers",
        "pl",
        "pneg",
        "pst",
        "post",
        "pot",
        "pp",
        "pprs",
        "pri",
        "prn",
        "punct",
        "px1pl",
        "px2pl",
        "px1sg",
        "px2sg",
        "px3sp",
        "qst",
        "qu",
        "rec",
        "reflex",
        "rel",
        "sg",
        "sp",
        "sup",
        "sym",
        "top",
        "tra",
        "use_dialect",
        "use_archaic",
        "use_nonstd",
        "use_foreign",
        "use_blacklist",
        "vaux",
        "vblex"
    }
    ## a mapping from omorfi stuff to apertium symbols
    stuff2apertium = {
        "Aiden": "",
        "Aien": "",
        "Aiin": "",
        "Ain": "",
        "Ayn": "",
        "Aän": "",
        "Aön": "",
        "Aisiin": "",
        "Aseen": "",
        "Aä": "",
        "Ajä": "",
        "Atä": "",
        "Ajen": "",
        "Aten": "",
        "Ahin": "",
        "Ahen": "",
        "Ahyn": "",
        "Aihin": "",
        "Aiä": "",
        "Ana": "",
        "Asa": "",
        "Aitten": "",
        "Aan": "",
        "Aen": "",
        "Ahan": "",
        "Ahon": "",
        "Ahun": "",
        "Aon": "",
        "Aun": "",
        "Aa": "",
        "Aia": "",
        "Aita": "",
        "Aja": "",
        "Ahän": "",
        "Ahön": "",
        "Aitä": "",
        "Ata": "",
        "ABBREVIATION": "abbr",
        "ACRONYM": "acr",
        "ADJ": "adj",
        "ADP": "post",
        "ADPOSITION": "post",
        "ADV": "adv",
        "ADVERB": "adv",
        "SCONJ": "cnjsub",
        "ADVERBIAL": "cnjadv",
        "AINF_arg": "vaux",
        "ARTWORK": "",
        "ARROW": "",
        "AUX": "vaux",
        "B-": "cmp-split",
        "Bxxx": "compound-only-L",
        "B←": "compound-only-L",
        "B→": "compound-R",
        "Bc": "cmp>+",
        "BLACKLISTED": "use_blacklist",
        "CARDINAL": "card",
        "Ccmp": "comp",
        "CLAUSE-BOUNDARY": "",
        "Cma": "agent",
        "Cmaton": "pneg",
        "Cnut": "pp",
        "COMMA": "",
        "COMPARATIVE": "cnjsub",
        "COMP": "",
        "CONJUNCTION": "",
        "CONJUNCTIONVERB": "cnjcoo><vblex",
        "CONJ": "cnjcoo",
        "CCONJ": "cnjcoo",
        "COORDINATING": "cnjcoo",
        "Cpos": "pst",
        "Csup": "sup",
        "CULTGRP": "al",
        "Cva": "pprs",
        "DASH": "",
        "DECIMAL": "",
        "DERSTI": "",
        "DERTTAIN": "",
        "DET": "det",
        "DEMONSTRATIVE": "dem",
        "DIGIT": "",
        "Din": "+in<n",
        "Din²": "",
        "Ds": "+s<n",
        "Dhko": "+hko<adj",
        "Disa": "+isa<adj",
        "Dllinen": "+llinen<n",
        "Dlainen": "+lainen<n",
        "Dla": "+la<n",
        "Dnen": "+nen<n",
        "Dtar": "+tar<n",
        "Dton": "+ton<adj",
        "Dmainen": "+mainen<adj",
        "Du": "+u<n",
        "Dtava": "+tava<adj",
        "Dma": "+ma<n",
        "Dinen": "+inen<n",
        "Dja": "+ja<n",
        "Dmpi": "",
        "Dmaisilla": "+maisilla<adv",
        "Dminen": "+minen<n",
        "Dnut": "+nut<adj",
        "Dtu": "+tu<adj",
        "Duus": "+uus<adj",
        "Dva": "+va<adj",
        "Dmaton": "+maton<adj",
        "Dttain": "+ttain<adv",
        "Dttaa": "+ttaa<vblex",
        "Dtattaa": "+tattaa<vblex",
        "Dtatuttaa": "+tatuttaa<vblex",
        "Dtuttaa": "+tuttaa<vblex",
        "Dsti": "+sti<adv",
        "EVENT": "",
        "FEMALE": "f",
        "FINAL-BRACKET": "",
        "FINAL-QUOTE": "",
        "FIRST": "ant",
        "FRACTION": "",
        "GEO": "top",
        "Ia": "infa",
        "Ie": "infe",
        "Ima": "infma",
        "Iminen": "infminen",
        "INDEFINITE": "ind",
        "INITIAL-BRACKET": "",
        "INITIAL-QUOTE": "",
        "INTRANSITIVE_arg": "vblex",
        "INTJ": "ij",
        "INTERROGATIVE": "itg",
        "LAST": "cog",
        "LEMMA-START": "",
        "LEMMA-END": "",
        "MALE": "m",
        "MAINF_arg": "vaux",
        "MEDIA": "al",
        "MISC": "al",
        "MULTIPLICATIVE": "",
        "Ncon": "conneg",
        "N??": "sp",
        "Nneg": "neg",
        "NOUN": "n",
        "Npl": "pl",
        "Nsg": "sg",
        "NUM": "num",
        "NUMERAL": "num",
        "O3": "px3sp",
        "Opl1": "px1pl",
        "Opl2": "px2pl",
        "ORDINAL": "ord",
        "ORD": "ord",
        "ORG": "al",
        "UNKNOWNPROPN": "al",
        "Osg1": "px1sg",
        "Osg2": "px2sg",
        "PARTICLE": "part",
        "PERSONAL": "pers",
        "PL1": "p1",
        "PL2": "p2",
        "PL3": "p3",
        "Ppe4": "impers",
        "Ppl1": "p1><pl",
        "Ppl2": "p2><pl",
        "Ppl3": "p3><pl",
        "PRON": "prn",
        "PRODUCT": "al",
        "PROPN": "np",
        "PROPER": "np",
        "Psg1": "p1><sg",
        "Psg2": "p2><sg",
        "Psg3": "p3><sg",
        "Psg0": "p3><sg",
        "PUNCT": "punct",
        "Qhan": "+han<enc",
        "Qkaan": "+kaan<enc",
        "Qka": "+ka<enc",
        "Qkin": "+kin<enc",
        "Qko": "+ko<qst",
        "Qpa": "+pa<enc",
        "Qs": "+s<enc",
        "QUALIFIER": "adj",
        "QUANTOR": "qu",
        "RECIPROCAL": "rec",
        "REFLEXIVE": "reflex",
        "RELATIVE": "rel",
        "ROMAN": "",
        ".sent": "",
        "SG1": "p1",
        "SG2": "p2",
        "SG3": "p3",
        "SG0": "p3",
        "SENTENCE-BOUNDARY": "",
        "SPACE": "",
        "SUFFIX": "",
        "SUPERL": "sup",
        "SYM": "sym",
        "Tcond": "cni",
        "Timp": "imp",
        "Topt": "",
        "Tpast": "past",
        "Tpot": "pot",
        "Tpres": "pri",
        "Uarch": "use_archaic",
        "Udial": "use_nonstd",
        "Unonstd": "use_nonstd",
        "UNSPECIFIED": "part",
        "Urare": "",
        "Vact": "actv",
        "VERB": "vblex",
        "Vpss": "pasv",
        "X???": "",
        "Xabe": "abe",
        "Xabl": "abl",
        "Xacc": "acc",
        "Xade": "ade",
        "Xall": "all",
        "Xcom": "com",
        "Xela": "ela",
        "Xess": "ess",
        "Xgen": "gen",
        "Xill": "ill",
        "Xine": "ine",
        "Xins": "ins",
        "Xlat": "lat",
        "Xnom": "nom",
        "Xpar": "par",
        "Xtra": "tra",
        "ADESSIVE": "ade",
        "ABLATIVE": "abl",
        "ALLATIVE": "all",
        "INESSIVE": "ine",
        "ILLATIVE": "ill",
        "LOCATIVE": "loc",
        "FTB3MAN": "",
        "FTB3man": "",
        ".": "",
        "XForeign": "use_foreign",
        "X": "x",
        "": ""
    }

    def __init__(self, verbose=True):
        """Create an apertium formatter with given verbosity.

        @param verbose  set to false to disable stdout logging
        """
        ## verbosity, i.e. print while translating
        super().__init__(verbose)
        self.verbose = verbose
        for _, ape in self.stuff2apertium.items():
            if len(ape) < 2:
                continue
            elif ape.startswith('+'):
                if not ape[ape.find('+'):]:
                    just_fail("There are conflicting formattings in here! " +
                              ape[ape.find('+'):] +
                              " is not a valid apertium multichar_symbol!")
            elif ape.endswith('+'):
                if not ape[:ape.find('+')]:
                    just_fail("There are conflicting formattings in here! " +
                              ape[ape.find('+'):] +
                              " is not a valid apertium multichar_symbol!")
            elif ape not in self.apertium_multichars:
                just_fail("There are conflicting formattings in here! " + ape +
                          " is not a valid apertium multichar_symbol!")

    def stuff2lexc(self, stuff):
        """Get apertium analyses in lexc format corresponding omorfi stuff.

        @return str containing lexc formatted analysis tag(s)
        """
        if not stuff:
            return ""
        elif stuff in self.stuff2apertium:
            if self.stuff2apertium[stuff] in ['+', '-', '#', '0', '']:
                return self.stuff2apertium[stuff]
            elif self.stuff2apertium[stuff].startswith('+'):
                return lexc_escape(self.stuff2apertium[stuff]) + '%>'
            elif self.stuff2apertium[stuff].endswith('+'):
                return '%<' + lexc_escape(self.stuff2apertium[stuff])
            else:
                return '%<' + lexc_escape(self.stuff2apertium[stuff]) + '%>'
        else:
            fail_formatting_missing_for(stuff, "apertium")
            return ""

    def analyses2lexc(self, anals, surf):
        """Get full analysis string in lexc for omorfi analyses.

        @return str containing lexc-formatted analysi tags and potential lemma
                fragments
        """
        apestring = ''
        for i in anals.split('|'):
            if i == '@@COPY-STEM@@':
                apestring += lexc_escape(surf)
            elif i.startswith('@@LITERAL') and i.endswith('@@'):
                apestring += lexc_escape(i[len('@@LITERAL'):-len('@@')])
            else:
                apestring += self.stuff2lexc(i)
        return apestring

    def continuation2lexc(self, anals, surf, cont):
        """Get lexc representation for omorfi continuation lexicon."""
        analstring = self.analyses2lexc(anals, surf)
        surf = lexc_escape(surf)
        return "%s:%s\t%s ;\n" % (analstring, surf, cont)

    def wordmap2lexc(self, wordmap):
        """Get lexc representation of omorfi lexeme in a wordmap.

        @return lexc-formatted entries for the word
        """
        if wordmap['lemma'] == ' ':
            # apertium fails when surf == ' '
            return ''
        wordmap['analysis'] = lexc_escape(wordmap['lemma'])
        wordmap['analysis'] = wordmap['analysis'].replace(
            word_boundary, '+').replace(weak_boundary, '')
        if wordmap['is_prefix']:
            wordmap['analysis'] += "+"
        elif wordmap['upos'] == 'PROPN':
            wordmap['analysis'] += self.stuff2lexc(wordmap['upos'])
            if wordmap['proper_noun_class']:
                wordmap['analysis'] +=\
                    self.stuff2lexc(wordmap['proper_noun_class'])
                if wordmap['sem'] in ['MALE', 'FEMALE']:
                    wordmap['analysis'] += self.stuff2lexc(wordmap['sem'])
            else:
                wordmap['analysis'] += self.stuff2lexc('UNKNOWNPROPN')
        elif wordmap['upos'] == 'VERB':
            if wordmap['argument']:
                wordmap[
                    'analysis'] += self.stuff2lexc(wordmap['argument'] +
                                                   '_arg')
            else:
                wordmap['analysis'] += self.stuff2lexc(wordmap['upos'])
        elif wordmap['upos'] == 'CCONJ|VERB':
            if wordmap['lemma'] == 'eikä':
                wordmap['lemma'] = 'ei'
                wordmap['analysis'] = 'ja' + \
                    self.stuff2lexc('COORDINATING') + \
                    '+ei' + \
                    self.stuff2lexc('Nneg')
            else:
                wordmap['analysis'] = wordmap['lemma'][:-2] + \
                    self.stuff2lexc('ADVERBIAL') + \
                    '+' + wordmap['lemma'][-2:] + \
                    self.stuff2lexc('Nneg')
        elif wordmap['particle']:
            for pclass in wordmap['particle'].split('|'):
                wordmap['analysis'] += self.stuff2lexc(pclass)
        else:
            wordmap['analysis'] += self.stuff2lexc(wordmap['upos'])
        if wordmap['blacklist']:
            if wordmap['blacklist'] != "TOOSHORTFORCOMPOUND":
                wordmap['analysis'] += self.stuff2lexc('BLACKLISTED')

        if wordmap['pronoun']:
            for stuff in wordmap['pronoun'].split("|"):
                wordmap['analysis'] += self.stuff2lexc(stuff)
        if wordmap['lex']:
            for stuff in wordmap['lex'].split("|"):
                wordmap['analysis'] += self.stuff2lexc(stuff)
        if wordmap['abbr']:
            for stuff in wordmap['abbr'].split("|"):
                wordmap['analysis'] += self.stuff2lexc(stuff)
        if wordmap['numtype']:
            for stuff in wordmap['numtype'].split("|"):
                wordmap['analysis'] += self.stuff2lexc(stuff)
        if wordmap['symbol']:
            for subcat in wordmap['symbol'].split('|'):
                wordmap['analysis'] += self.stuff2lexc(subcat)
            if wordmap['stub'] in ";:":
                wordmap['analysis'] += self.stuff2lexc("SENTENCE-BOUNDARY")
        # XXX: for now
        if wordmap['lemma'] in "¹²³½¼=≥µ#/%":
            wordmap['analysis'] += self.stuff2lexc("NOUN")
        wordmap['stub'] = wordmap['stub']
        wordmap['stub'] = lexc_escape(wordmap['stub'])
        if 'BLACKLIST' in wordmap['new_para']:
            return "!%s:%s\t%s\t;" % (wordmap['analysis'], wordmap['stub'],
                                      wordmap['new_para'])
        else:
            return "%s:%s\t%s\t;" % (wordmap['analysis'], wordmap['stub'],
                                     wordmap['new_para'])

    def multichars_lexc(self):
        """Get apertium compatible lexc multichars.

        @return str containing lexc multichar_symbols block for apertium tags
        """
        multichars = "Multichar_Symbols\n!! Apertium standard tags:\n"
        for mcs in sorted(self.apertium_multichars):
            if '><' not in mcs and mcs not in ['', '+', '-', '#', '0']:
                multichars += '%<' + lexc_escape(mcs) + "%>\n"
        multichars += Formatter.multichars_lexc(self)
        return multichars

    def root_lexicon_lexc(self):
        """Get apertium compatible lexc root.

        @return str containing lexc Root lexicon for apertium"""
        root = Formatter.root_lexicon_lexc(self)
        root += '\t'.join(['0', 'SUFFIX', ';']) + '\n'
        root += '\t'.join(['-', 'NOUN', ';']) + '\n'
        root += '\t'.join(['-', 'ADJ', ';']) + '\n'
        return root


# self test
if __name__ == '__main__':
    formatter = ApertiumFormatter()
    exit(0)
